/* bibcite is a bibliography and citation library with exporting
and importing capabilities to bibTex format

Copyright (C) 2010  Georg Adelmann - mail@geoadel.net

This file is part of bibcite.

bibcite free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>. */
package net.geoadel.bibcite.Cryptography;

import java.io.IOException;
import java.security.InvalidKeyException;
import java.security.KeyFactory;
import java.security.KeyPair;
import java.security.KeyPairGenerator;
import java.security.NoSuchAlgorithmException;
import java.security.PrivateKey;
import java.security.PublicKey;
import java.security.spec.EncodedKeySpec;
import java.security.spec.InvalidKeySpecException;
import java.security.spec.PKCS8EncodedKeySpec;
import java.util.UUID;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.crypto.BadPaddingException;
import javax.crypto.Cipher;
import javax.crypto.IllegalBlockSizeException;
import javax.crypto.NoSuchPaddingException;
import javax.crypto.SealedObject;
import net.geoadel.bibcite.DataFields.SessionData;
import org.jasypt.encryption.pbe.StandardPBEByteEncryptor;
import org.jasypt.encryption.pbe.StandardPBEStringEncryptor;
import org.jasypt.exceptions.EncryptionOperationNotPossibleException;
import org.jasypt.encryption.StringEncryptor;
import org.jasypt.util.text.BasicTextEncryptor;

/**
 * Four password levels are used for encryption / decryption
 *
 * implements www.jasypt.org standard for encrypting and decrypting text and byte streams (files)
 * @GPLStatus   ready
 * @DocumentationStatus finished
 * @author Georg Adelmann
 * @urlJasypt http://www.jasypt.org
 */
public class EncryptDecrypt {

    static StandardPBEStringEncryptor stringEncryptor = new StandardPBEStringEncryptor();
    static BasicTextEncryptor textEncryptor = new BasicTextEncryptor();
    static StandardPBEByteEncryptor fileEncryptor = new StandardPBEByteEncryptor();
    static Cipher cipher;

    /*
     * PUBLIC & PRIVATE KEY used to encrypt / decrypt object passwords
     */
    /**
     *
     * @return  KeyPair with private and public key
     */
    public KeyPair createKeyPair() {
        try {
            KeyPairGenerator keyPairGenerator = KeyPairGenerator.getInstance("RSA");
            keyPairGenerator.initialize(512);
            KeyPair keyPair = keyPairGenerator.generateKeyPair();
            return keyPair;
        } catch (NoSuchAlgorithmException ex) {
            Logger.getLogger(EncryptDecrypt.class.getName()).log(Level.SEVERE, null, ex);
        }
        return null;
    }

    /**
     *
     * @param privateKeyBytes   encrypted byte array (using login password) of the private key
     * @return  recreated PrivateKey from the encrypted byte array
     */
    public PrivateKey recreatePrivateKeyFromBytes(byte[] privateKeyBytes) {
        try {
            fileEncryptor = new StandardPBEByteEncryptor();
            fileEncryptor.setPassword(SessionData.getPasswordUserLogin());
            byte[] decryptedBytes = fileEncryptor.decrypt(privateKeyBytes);

            KeyFactory keyFactory = KeyFactory.getInstance("RSA");
            EncodedKeySpec privateKeySpec = new PKCS8EncodedKeySpec(decryptedBytes);
            return keyFactory.generatePrivate(privateKeySpec);
        } catch (NoSuchAlgorithmException ex) {
            Logger.getLogger(EncryptDecrypt.class.getName()).log(Level.SEVERE, null, ex);
        } catch (InvalidKeySpecException ex) {
            Logger.getLogger(EncryptDecrypt.class.getName()).log(Level.SEVERE, null, ex);
        }
        return null;
    }

    /**
     *
     * @param privateKey
     * @return  byte array representing the encrypted private key
     */
    public byte[] encryptPrivateKey(PrivateKey privateKey) {
        fileEncryptor = new StandardPBEByteEncryptor();
        fileEncryptor.setPassword(SessionData.getPasswordUserLogin());
        byte[] encryptedBytes = fileEncryptor.encrypt(privateKey.getEncoded());
        return encryptedBytes;
    }

    /*
     * /PUBLIC & PRIVATE KEY used to encrypt / decrypt object passwords
     */

    /*
     * OBJECT PASSWORD used to encrypt / decrypt database object fields, encrypted by PublicKey, needs PrivateKey for decryption
     */
    /**
     *
     * @param privateObjectPassword     password for private object
     * @param publicKey                 users public encryption key used for encrypting the password of private object (needs his private key to decrypt the password)
     * @return                          private object decryption password stored in a sealed container encrypted by user public key
     */
    public SealedObject encryptPrivateObjectPassword(String privateObjectPassword, PublicKey publicKey) {
        try {

            if (cipher == null) {
                cipher = Cipher.getInstance("RSA");
            }
            cipher.init(Cipher.ENCRYPT_MODE, publicKey);
            return new SealedObject(privateObjectPassword, cipher);
        } catch (IOException ex) {
            Logger.getLogger(EncryptDecrypt.class.getName()).log(Level.SEVERE, null, ex);
        } catch (IllegalBlockSizeException ex) {
            Logger.getLogger(EncryptDecrypt.class.getName()).log(Level.SEVERE, null, ex);
        } catch (InvalidKeyException ex) {
            Logger.getLogger(EncryptDecrypt.class.getName()).log(Level.SEVERE, null, ex);
        } catch (NoSuchAlgorithmException ex) {
            Logger.getLogger(EncryptDecrypt.class.getName()).log(Level.SEVERE, null, ex);
        } catch (NoSuchPaddingException ex) {
            Logger.getLogger(EncryptDecrypt.class.getName()).log(Level.SEVERE, null, ex);
        }
        return null;
    }

    /**
     *
     * @param sealedObject
     * @param privateKey
     * @return
     */
    public String decryptPrivateObjectPassword(SealedObject sealedObject, PrivateKey privateKey) {
        try {
            if (cipher == null) {
                cipher = Cipher.getInstance("RSA");
            }
            cipher.init(Cipher.DECRYPT_MODE, privateKey);
            return (String) sealedObject.getObject(cipher);
        } catch (IOException ex) {
            Logger.getLogger(EncryptDecrypt.class.getName()).log(Level.SEVERE, null, ex);
        } catch (ClassNotFoundException ex) {
            Logger.getLogger(EncryptDecrypt.class.getName()).log(Level.SEVERE, null, ex);
        } catch (IllegalBlockSizeException ex) {
            Logger.getLogger(EncryptDecrypt.class.getName()).log(Level.SEVERE, null, ex);
        } catch (BadPaddingException ex) {
            Logger.getLogger(EncryptDecrypt.class.getName()).log(Level.SEVERE, null, ex);
        } catch (InvalidKeyException ex) {
            Logger.getLogger(EncryptDecrypt.class.getName()).log(Level.SEVERE, null, ex);
        } catch (NoSuchAlgorithmException ex) {
            Logger.getLogger(EncryptDecrypt.class.getName()).log(Level.SEVERE, null, ex);
        } catch (NoSuchPaddingException ex) {
            Logger.getLogger(EncryptDecrypt.class.getName()).log(Level.SEVERE, null, ex);
        }
        return null;
    }


    /*
     * / OBJECT PASSWORD used to encrypt / decrypt database object fields, encrypted by PublicKey, needs PrivateKey for decryption
     */

    /*
     * DECRYPT / ENCRYPT OBJECT FIELDS encrypted byobject password
     */
    /**
     * encrypts strings using another string as password and returns the encrypted String
     * @param text  the text to be encrypted
     * @return  the encrypted text
     */
    public String encryptPrivate(String text, UUID uuid) {
        textEncryptor = new BasicTextEncryptor();
        textEncryptor.setPassword(SessionData.getObjectFieldsPassword(uuid));

        // System.out.println("Password for encryption: "+SessionData.getObjectFieldsPassword(uuid));

        String encryptedString = textEncryptor.encrypt(text);
        // System.out.println("PRIVATE text "+text + " Password for encryption: "+SessionData.getObjectFieldsPassword(uuid));
        // System.out.println("     ->    "+encryptedString);
        return encryptedString;
    }

    /**
     * encrypts byte arrays using the permanent user password
     * @param decryptedBytes    the decrypted byte array to be encrypted
     * @return  the encrypted byte array
     */
    public byte[] encryptPrivate(byte[] decryptedBytes, UUID uuid) {
        // System.out.println("Encrypting private byte array");
        fileEncryptor = new StandardPBEByteEncryptor();
        fileEncryptor.setPassword(SessionData.getObjectFieldsPassword(uuid));
        byte[] encryptedBytes = fileEncryptor.encrypt(decryptedBytes);
        return encryptedBytes;
    }

    /**
     * encrypts strings using another string as password and returns the encrypted String
     * @param text  the text to be encrypted
     * @return  the encrypted text
     */
    public String encryptPublic(String text) {
        textEncryptor = new BasicTextEncryptor();
        textEncryptor.setPassword(SessionData.getPasswordProgramPermanent());
        String encryptedText = textEncryptor.encrypt(text);
        // System.out.println("PUBLIC Text "+text+" Encrypted: "+encryptedText);
        return encryptedText;
    }

    /**
     * encrypts byte arrays using the universal program password
     * @param decryptedBytes    the decrypted byte array to be encrypted
     * @return  the encrypted byte array
     */
    public byte[] encryptPublic(byte[] decryptedBytes) {
        StandardPBEByteEncryptor fileEncryptor = new StandardPBEByteEncryptor();
        fileEncryptor.setPassword(SessionData.getPasswordProgramPermanent());
        byte[] encryptedBytes = fileEncryptor.encrypt(decryptedBytes);
        return encryptedBytes;
    }

    public String decryptPrivate(String encryptedText, UUID uuid) {
        BasicTextEncryptor textEncryptor = new BasicTextEncryptor();
        textEncryptor.setPassword(SessionData.getObjectFieldsPassword(uuid));
        // System.out.println("Using private password: "+SessionData.getObjectFieldsPassword(uuid));
        String decryptedText = "";
        try{
            decryptedText = textEncryptor.decrypt(encryptedText);
        } catch(EncryptionOperationNotPossibleException ex){
            // System.out.println("ERROR: "+encryptedText);
            ex.printStackTrace();
        }
        // System.out.println("     ->    "+decryptedText);
        return decryptedText;
    }

    public byte[] decryptPrivate(byte[] encryptedBytes, UUID uuid) {
        StandardPBEByteEncryptor fileEncryptor = new StandardPBEByteEncryptor();
        fileEncryptor.setPassword(SessionData.getObjectFieldsPassword(uuid));
        byte[] decryptedBytes = fileEncryptor.decrypt(encryptedBytes);
        return decryptedBytes;
    }

    public String decryptPublic(String encryptedText) {
        textEncryptor = new BasicTextEncryptor();
        textEncryptor.setPassword(SessionData.getPasswordProgramPermanent());
        // System.out.println("PUBLIC text "+encryptedText + " Password for decryption: "+SessionData.getPasswordProgramPermanent());
        String decryptedText = textEncryptor.decrypt(encryptedText);
        // System.out.println("     ->    "+decryptedText);
        return decryptedText;
    }

    public byte[] decryptPublic(byte[] encryptedBytes) {
        StandardPBEByteEncryptor fileEncryptor = new StandardPBEByteEncryptor();
        fileEncryptor.setPassword(SessionData.getPasswordProgramPermanent());
        byte[] decryptedBytes = fileEncryptor.decrypt(encryptedBytes);
        return decryptedBytes;
    }
}
