/* bibcite is a bibliography and citation library with exporting
and importing capabilities to bibTex format

Copyright (C) 2010  Georg Adelmann - mail@geoadel.net

This file is part of bibcite.

bibcite free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>. */
package net.geoadel.bibcite.Database;

import com.db4o.Db4oEmbedded;
import com.db4o.ObjectContainer;
import com.db4o.config.EmbeddedConfiguration;
import com.db4o.query.Predicate;
import java.io.File;
import net.geoadel.bibcite.Tools.FileOperations;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.UUID;

import net.geoadel.bibcite.DatabaseObjects.*;
import net.geoadel.bibcite.DataFields.Preferences;
import net.geoadel.bibcite.DataFields.SessionData;
import net.geoadel.bibcite.DatabaseObjects.Note;
import net.geoadel.bibcite.DatabaseObjects.dbObject;
import net.geoadel.bibcite.Tools.ObjectTools;

/**
 * db4o.java provides read write access to the db4o database
 * @urlDb4o http:/www.db4o.com
 *
 * @DocumentationStatus finished
 * @GPLStatus clean
 * @author Georg Adelmann
 */
public class db4o {

    static ObjectContainer objC;
    static ObjectContainer userC;

    /**
     * db_access returns an object container pointing at the harddrive file containing the object database
     * @return
     */
    public void db_access(String databasePath) {
        // System.out.println("Accessing database at "+databasePath);
        if (objC == null) {
            // Configure access database details
            EmbeddedConfiguration configData = Db4oEmbedded.newConfiguration();
            // set the updat depth for user class to two, so also embedded objects
            // like HashMaps are updated correctly
            configData.common().updateDepth(3);
            // accessDb4o
            new File(databasePath).mkdirs();
            ObjectContainer db = Db4oEmbedded.openFile(configData, databasePath + "DBBib.db4o"); // this is the access command for db40 7.4
            db4o.objC = db;
        }

        if (userC == null) {
            EmbeddedConfiguration configUser = Db4oEmbedded.newConfiguration();

            configUser.common().updateDepth(3);
            new File(databasePath).mkdirs();
            ObjectContainer dbUser = Db4oEmbedded.openFile(configUser, databasePath + "DBUser.db4o"); // this is the access command for db40 7.4
            db4o.userC = dbUser;
        }
    }

    /**
     * Erase preexisting database in pathDatabase - only used for debugging purposes
     * @param pathDatabase 
     */
    public void db_reset(String pathDatabase) {
        Preferences pref = new Preferences();
        File fObjects = new File(pathDatabase + "DBBib.db4o");
        File fUsers = new File(pathDatabase + "DBUser.db4o");
        System.out.println(fObjects.exists());
        fObjects.delete();
        fUsers.delete();
        System.out.println(fObjects.exists());
    }

    public void storeObject(final dbObject object, User user) {
        // System.out.println("user has access: " + user.hasAccessToPrivateObject(bib.getUUID()));
        // System.out.println("SessionData.getUser() has access: " + SessionData.getUser().hasAccessToPrivateObject(bib.getUUID()));

        // test whether this element with same revision date and uuid is already in database, if so, skip storing it - can happen with multiple imports for example
        List<dbObject> result = db4o.objC.query(new Predicate<dbObject>() {

            public boolean match(dbObject search) {
                // for some strange reason in some cases of imported bibliographies, the uuid comparison only works reliably if they are in string format
                return search.getUUID().equals(object.getUUID()) & search.getDateEdited().equals(object.getDateEdited());
            }
        });

        if (result.isEmpty()) {
            // System.out.println("Storing bib " + bib.getUUID() + " since no other element with samge uuid and revision date was found");
            db4o.objC.store(object);
            if(object instanceof Bibliography){
                System.out.println("Stored bib from "+((Bibliography)object).getDateEdited()+" with booktitle "+((Bibliography)object).getBooktitle());
            }
            // System.out.println("Stored bib " + getBibliographyByUUID(bib.getUUID(), user));
        } else {
            // System.out.println("Element with same uuid and revision date already in database - skip storing.");
        }
    }

    /**
     * get all objects, this user has access to. Needed e.g. for RelationManager
     * @param user
     * @return
     */
    public ArrayList<dbObject> getObjects(final User user) {
        List<dbObject> result = db4o.objC.query(new Predicate<dbObject>() {

            public boolean match(dbObject search) {
                // System.out.println("User "+ user.getName()+ " has access to "+ search.getName()+": "+user.hasAccessToPrivateObject(uuid));
                return !search.isPrivate() || user.hasAccessToPrivateObject(search.getUUID());
            }
        });

        ArrayList<dbObject> alReturn = new ArrayList<dbObject>();
        for (int i = 0; i < result.size(); i++) {
            alReturn.add(result.get(i));
        }

        alReturn = ObjectTools.filterNewestByUUID(alReturn);
        // System.out.println("retrieveBibliographyByUUID(final UUID " + uuid + ") closed database. Returned " + alReturn.size() + " entries.");
        return alReturn;
    }

    /**
     * get all objects with the given UUID from the database
     * @param uuid  uuid of the object requested
     * @param user  current user, in case, the bibliography is private and the string fields need to be decrypted (also, if it is private and the user is not the owner, it is not returned at all)
     * @return  ArrayList containing all the bibliographies with given uuid, that given user has access to
     */
    public ArrayList<dbObject> getObjectsByUUID(final UUID uuid, final User user) {

        List<dbObject> result = db4o.objC.query(new Predicate<dbObject>() {

            public boolean match(dbObject search) {
                return search.getUUID().equals(uuid) & (!search.isPrivate() || user.hasAccessToPrivateObject(uuid));
            }
        });

        ArrayList<dbObject> alReturn = new ArrayList<dbObject>();
        for (int i = 0; i < result.size(); i++) {
            alReturn.add(result.get(i));
        }

        return alReturn;
    }

    /**
     * Returns object with given uuid and date, if it exists and user is allowed access to it (either not private, or user is owner)
     * @param uuid  uuid of the object requested
     * @param date  specific date (revision) to be getd
     * @param user  user requesting the bibliography
     * @return  return all the objects with given date, uuid and user (should typically be only one, collisions are likely to occur with multiple imports from the same backup or source though)
     */
    public ArrayList<dbObject> getObjectByDateUUID(final UUID uuid, final long date, final User user) {
        List<dbObject> result = db4o.objC.query(new Predicate<dbObject>() {

            public boolean match(dbObject search) {
                return search.getUUID().equals(uuid) & search.getDateEdited() == date & (!search.isPrivate() || user.hasAccessToPrivateObject(uuid));
            }
        });

        ArrayList<dbObject> vReturn = new ArrayList<dbObject>();
        for (dbObject object : result) {
            vReturn.add(object);
        }
        return vReturn;
    }

    /**
     * gets all bibliographies (and all their revisions), that the user is granted access to
     * @param user
     * @param db
     * @return
     */
    public ArrayList<Bibliography> getBibliographiesAll(final User user) {
        ArrayList<Bibliography> alReturn = new ArrayList<Bibliography>();
        for (dbObject obj : getObjects(user)) {
            if (obj instanceof Bibliography) {
                alReturn.add((Bibliography) obj);
            }
        }
        return alReturn;
    }

    /**
     * gets all quotations (including all revisions of the same uuid) the user is granted access to
     * @param db    database connection, if method should not create its own
     * @param user  user, that is trying to get all quotations
     * @return  returns ArrayList of all found quotations
     */
    public ArrayList<Quotation> getQuotationsAll(final User user) {
        ArrayList<Quotation> alReturn = new ArrayList<Quotation>();
        for (dbObject obj : getObjects(user)) {
            if (obj instanceof Quotation) {
                alReturn.add((Quotation) obj);
            }
        }
        return alReturn;
    }

    /**
     * gets all quotations (including all revisions of the same uuid) the user is granted access to
     * @param db    database connection, if method should not create its own
     * @param user  user, that is trying to get all quotations
     * @return  returns ArrayList of all found quotations
     */
    public ArrayList<Person> getPersonsAll(final User user) {
        ArrayList<Person> alReturn = new ArrayList<Person>();
        for (dbObject obj : getObjects(user)) {
            if (obj instanceof Person) {
                alReturn.add((Person) obj);
            }
        }
        return alReturn;
    }

    /**
     * gets all quotations (including all revisions of the same uuid) the user is granted access to
     * @param db    database connection, if method should not create its own
     * @param user  user, that is trying to get all quotations
     * @return  returns ArrayList of all found quotations
     */
    public ArrayList<Note> getNotesAll(final User user) {
        ArrayList<Note> alReturn = new ArrayList<Note>();
        for (dbObject obj : getObjects(user)) {
            if (obj instanceof Note) {
                alReturn.add((Note) obj);
            }
        }
        return alReturn;
    }

    /**
     * gets all quotations (including all revisions of the same uuid) the user is granted access to
     * @param db    database connection, if method should not create its own
     * @param user  user, that is trying to get all quotations
     * @return  returns ArrayList of all found quotations
     */
    public ArrayList<dbFile> getFilesAll(final User user) {
        ArrayList<dbFile> alReturn = new ArrayList<dbFile>();
        for (dbObject obj : getObjects(user)) {
            if (obj instanceof dbFile) {
                alReturn.add((dbFile) obj);
            }
        }
        return alReturn;
    }

    /**
     * gets the newest version of all bibliographies, the user is granted access to
     * @param user  the current user asking for the bibliographies
     * @param db    a database connection, if method should not open one for itself
     * @return  returns only the newest bibliography entries for each uuid - useful for exporting to other formats - NOT useful for backups, since older revisions get lost
     */
    public ArrayList<Bibliography> getBibliographiesNewestVersions(final User user) {
        ArrayList<Bibliography> alReturn = new ArrayList<Bibliography>();
        ArrayList<Bibliography> alBibliograhpies = getBibliographiesAll(user);
        ArrayList<dbObject> familieMembersNewest = ObjectTools.filterNewestByUUID(ObjectTools.bibToDBObject(alBibliograhpies));
        for (dbObject object : familieMembersNewest) {
            if (object instanceof Bibliography) {
                alReturn.add((Bibliography) object);
            }
        }
        return alReturn;
    }

    /**
     * gets for each UUID the one with the highest dateEdited Long value - aka the newest version
     * @param user      the logged in user - needed for decrypting private strings
     * @param db        the database access, if this method should not use its own
     * @return          only the newest versions of each uuid, if not private or owned by this user
     */
    public ArrayList<Quotation> getQuotationsNewestVersions(final User user) {
        ArrayList<Quotation> alReturn = new ArrayList<Quotation>();
        ArrayList<Quotation> alQuotes = getQuotationsAll(user);
        ArrayList<dbObject> familieMembersNewest = ObjectTools.filterNewestByUUID(ObjectTools.quoteToDBObject(alQuotes));
        for (dbObject object : familieMembersNewest) {
            if (object instanceof Quotation) {
                alReturn.add((Quotation) object);
            }
        }
        return alReturn;
    }

    public ArrayList<Person> getPersonsNewestVersions(User user) {
        ArrayList<Person> alReturn = new ArrayList<Person>();
        ArrayList<Person> alPersons = getPersonsAll(user);
        ArrayList<dbObject> familieMembersNewest = ObjectTools.filterNewestByUUID(ObjectTools.personToDBObject(alPersons));
        for (dbObject object : familieMembersNewest) {
            if (object instanceof Person) {
                alReturn.add((Person) object);
            }
        }
        return alReturn;
    }

    public ArrayList<dbFile> getFilesNewestVersions(User user) {
        ArrayList<dbFile> alReturn = new ArrayList<dbFile>();
        ArrayList<dbFile> alFiles = getFilesAll(user);
        ArrayList<dbObject> familieMembersNewest = ObjectTools.filterNewestByUUID(ObjectTools.fileToDBObject(alFiles));
        for (dbObject object : familieMembersNewest) {
            if (object instanceof dbFile) {
                alReturn.add((dbFile) object);
            }
        }
        return alReturn;
    }

    public ArrayList<Note> getNotesNewestVersions(User user) {
        ArrayList<Note> alReturn = new ArrayList<Note>();
        ArrayList<Note> alNotes = getNotesAll(user);
        ArrayList<dbObject> familieMembersNewest = ObjectTools.filterNewestByUUID(ObjectTools.noteToDBObject(alNotes));
        for (dbObject object : familieMembersNewest) {
            if (object instanceof Note) {
                alReturn.add((Note) object);
            }
        }
        return alReturn;
    }

    /**
     * gets all quotations related to the given bibliography uuid
     * @param uuid  uuid of the bibliography that the quotation is related to
     * @param user   the user, that is trying to get the data
     * @return  returns the newest entries for each getd uuid
     */
    public ArrayList<Quotation> getQuotationsByBibUUID(final UUID uuid, final User user) {
        ArrayList<Quotation> alQuotes = getQuotationsAll(user);
        ArrayList<dbObject> newestQuotesPerFamily = ObjectTools.filterNewestByUUID(ObjectTools.quoteToDBObject(alQuotes));
        ArrayList<Quotation> alResult = new ArrayList<Quotation>();
        for (dbObject object : newestQuotesPerFamily) {
            if (object.getRelatedObjects().contains(uuid)) {
                alResult.add((Quotation) object);
            }
        }
        return alResult;
    }

    /**
     * gets all nots related to the given uuid
     * @param uuid  uuid of the bibliography that the quotation is related to
     * @param user   the user, that is trying to get the data
     * @return  returns the newest entries for each getd uuid
     */
    public ArrayList<Note> getNotesRelatedToUUID(final UUID sourceUUID, final User user) {
        ArrayList<dbObject> alObjects = getObjects(user);
        ArrayList<dbObject> newestObjectPerFamily = ObjectTools.filterNewestByUUID(alObjects);

        ArrayList<Note> alReturn = new ArrayList<Note>();
        for (dbObject object : newestObjectPerFamily) {
            if (object.getRelatedObjects().contains(sourceUUID) & object instanceof Note) {
                alReturn.add((Note) object);
            }
        }
        return alReturn;
    }

    /**
     * get all bibliography keys (e.g. used to create unique keys that do not collide with preexisting keys
     * note, that this does not filter for the current user (though actually for key generation, this is tricky, since it could create colliding keys, if two users added the same bibliography and one was private)
     * but since private keys are encrypted anyway, the collision cannot be helped in the current design
     * @return  ArrayList<String> containing all keys of bibliographies the user is granted access to
     */
    public ArrayList<String> getKeys(User user) {
        ArrayList<Bibliography> bibs = getBibliographiesNewestVersions(user);
        ArrayList<String> vKeys = new ArrayList<String>();
        for (int i = 0; i < bibs.size(); i++) {
            if (!vKeys.contains(bibs.get(i).getKey())) {
                vKeys.add(bibs.get(i).getKey());
            }
        }
        return vKeys;
    }

    /**
     * get all bibliography keys (e.g. used to create unique keys that do not collide with preexisting keys
     * note, that this does not filter for the current user (though actually for key generation, this is tricky, since it could create colliding keys, if two users added the same bibliography and one was private)
     * but since private keys are encrypted anyway, the collision cannot be helped in the current design
     * @return  ArrayList<String> containing all keys of bibliographies the user is granted access to
     */
    public ArrayList<String> getQuotationKeys(User user) {
        ArrayList<Quotation> quotes = getQuotationsAll(user);
        ArrayList<String> vKeys = new ArrayList<String>();
        for (int i = 0; i < quotes.size(); i++) {
            if (!vKeys.contains(quotes.get(i).getKey())) {
                vKeys.add(quotes.get(i).getKey());
            }
        }
        return vKeys;
    }

    /**
     * store a bunch of bibliographies at a time (used e.g. for importing bibliographies)
     * @param alBibliographies  ArrayList containing the bibliographies to be stored
     * @param user  the user trying to store the bibliographies
     */
    public void storeObjects(ArrayList<dbObject> alObjects, User user) {
        for (dbObject object : alObjects) {
            storeObject(object, user);
        }
    }

    /**
     * Erase a bibliography
     * @param uuid  uuid of the bibliography to be erased
     * @param user  user is used to check, that the user actually has access to the bibliography so he is entitled to delete it - actually he cannot see the bibliography, so this safety check is probably moot
     */
    public void eraseObject(final UUID uuid, final User user) {
        ArrayList<dbObject> alObjects = getObjectsByUUID(uuid, user);
        for (dbObject object : alObjects) {
            objC.delete(object);
        }
    }

    /**
     * Store a file in database and in objects uuid directory on the physical harddrive
     * @param file  the file to be stored
     * @param db    database connection, if method should not create its own
     */
    public void storeFile(final dbFile file, final User user) {
        // check whether same file with same uuid and revision date already exists and delete it before writing the new one
        List<dbFile> result = db4o.objC.query(new Predicate<dbFile>() {

            public boolean match(dbFile search) {
                return search.getUUID().equals(file.getUUID()) & search.getRevision().equals(file.getRevision()) & (!search.isPrivate() || user.hasAccessToPrivateObject(search.getUUID()));
            }
        });

        if (!result.isEmpty()) { // delete the dbFiles with the same uuid AND revision date
            for (dbFile f : result) {
                db4o.objC.delete(f);
            }
        }

        // System.out.println("Storing file " + file.getName() + " UUID: " + file.getUUID());
        // store file in database
        db4o.objC.store(file);

        storeUser(user);
    }

    /**
     * store multiple files in one batch
     * @param files the files to be stored
     * @param db    database connection, if storeFile (the called method for each file) should not create its own connection
     */
    public void storeFiles(ArrayList<dbFile> files, User user) {
        for (int i = 0; i < files.size(); i++) {
            storeFile(files.get(i), user);
        }
    }

    /**
     * Erase a file from database and harddrive
     * @param fileUUID  uuid of file to be erased
     * @param user      user who is calling the method
     */
    public void eraseFile(final UUID fileUUID, final User user) {
        List<dbFile> result = db4o.objC.query(new Predicate<dbFile>() {

            public boolean match(dbFile file) {
                return file.getUUID().equals(fileUUID) & (!file.isPrivate() || user.hasAccessToPrivateObject(fileUUID));
            }
        });

        for (int i = 0; i < result.size(); i++) {
            if (result.get(i).getFile().exists()) {
                FileOperations.eraseFile(fileUUID, result.get(i));
            }
            // System.out.println(i + ": Deleting file " + result.get(i).getName());
            db4o.objC.delete(result.get(i));
        }
    }

    /**
     * Get all files related to uuid
     * @param sourceUUID    the uuid, to which files may be related
     * @param db    database connection, if method should not create its own
     * @return  ArrayList with related dbFiles
     */
    public ArrayList<dbFile> getFilesRelated(final UUID sourceUUID, final User user) {
        ArrayList<dbFile> alReturn = new ArrayList<dbFile>();

        List<dbFile> result = db4o.objC.query(new Predicate<dbFile>() {

            public boolean match(dbFile file) {
                return file.getRelatedObjects().contains(sourceUUID) & (!file.isPrivate() || user.hasAccessToPrivateObject(file.getUUID()));
            }
        });

        ArrayList<dbFile> alFiles = new ArrayList<dbFile>();
        for (dbFile f : result) {
            alFiles.add(f);
        }

        ArrayList<UUID> alFamilies = ObjectTools.filterFamiliesByUUID(ObjectTools.fileToDBObject(alFiles));
        Iterator<UUID> iFamilies = alFamilies.iterator();
        // for all uuid families
        while (iFamilies.hasNext()) {
            // get the next uuid family
            UUID familyUUID = iFamilies.next();
            ArrayList<dbFile> alCurrentFamily = new ArrayList<dbFile>();
            for (int i = 0; i < alFiles.size(); i++) {
                if (familyUUID.equals(alFiles.get(i).getUUID())) {
                    // find and add all members of the uuid family
                    alCurrentFamily.add(alFiles.get(i));
                }
            }
            // find the newest member of the uuid family and add it to the alReturn ArrayList
            int iNewest = ObjectTools.filterNewest(ObjectTools.fileToDBObject(alCurrentFamily));
            alReturn.add(alCurrentFamily.get(iNewest));
        }
        return alReturn;
    }

    /**
     * get the logged in user, if credentials (username and password) were matching an entry in the database
     * @param username  given username in login / register UI
     * @param password  given password in login / register UI
     * @return  User or null, if no user with given parameters was found in user database
     */
    public User userLogin(final String username, final String password) {

        // System.out.println("Checking if username " + username + " exists.");
        // accessDb4o

        // ObjectContainer db = Db4oEmbedded.openFile(Db4oEmbedded.newConfiguration(), "DBUser.db4o");
        // System.out.println("**** userLogin(" + username + ", "+password+") opened user database connection...");
        // Find all users with the given username (should only be one or none, since register-dialogue checks for collisions)
        List<User> result = userC.query(new Predicate<User>() {

            public boolean match(User user) {
                return user.getName().equals(username);
            }
        });

        // put all users with this password and name into alUser ArrayList
        ArrayList<User> alUser = new ArrayList<User>();
        for (User user : result) {
            alUser.add(user);
            // System.out.println("Found " + user);
        }

        User user = null;
        if (result.size() > 1) {
            // find newest user entry
            user = alUser.get(net.geoadel.bibcite.Tools.ObjectTools.filterNewest(net.geoadel.bibcite.Tools.ObjectTools.userToDBObject(alUser)));
        }
        if (result.size() < 1) {
            return null;
        }
        if (result.size() == 1) {
            user = result.get(0);
        }

        // test newest user, if user has the given password | if password were testes in match() method, then old passwords would be accepted since for old user entries with the same name and the old password, a match would be found
        if (user.isUser(password)) {

            // System.out.println("Logged in as user "+user);

            return user;
        } else {
            return null;
        }
    }

    /**
     * Store new user with name password and preferred language in database and return new dbUser object
     * @param name  name of new user
     * @param password  password of new user
     * @param language  preferred language of new user
     * @return  new User object
     */
    public User userRegister(String name, String password, String language) {
        // ObjectContainer db = Db4oEmbedded.openFile(Db4oEmbedded.newConfiguration(), "DBUser.db4o");
        User user = new User(name, password, language);
        userC.store(user);
        // System.out.println("Registered user "+name);
        return user;
    }

    /**
     * get a list of all usernames - used by login / register UI to avoid collisions of usernames
     * @return  ArrayList<String> containing all registered users in user database
     */
    public ArrayList<String> getUserNameList() {
        ArrayList<String> userList = new ArrayList<String>();

        // ObjectContainer db = Db4oEmbedded.openFile(Db4oEmbedded.newConfiguration(), "DBUser.db4o");
        List<User> result = userC.query(new Predicate<User>() {

            public boolean match(User user) {
                return true;
            }
        });
        Iterator<User> iResult = result.iterator();
        while (iResult.hasNext()) {
            userList.add(iResult.next().getName());
        }
        return userList;
    }

    /**
     * Return search result for all quotations and bibliographies containing all of the given keywords. All string fields are included in the search.
     * @param searchParameters  String parameters searched for
     * @param user  user that is conducting the search
     * @return  Object[] containing bibliographies[0] and quotations[1] that matched given search parameters
     */
    public Object[] returnSearchResults(ArrayList<String> searchParameters, User user) {
        Object[] returnObject = new Object[5];

        ArrayList<Bibliography> alBibs = getBibliographiesNewestVersions(user);
        ArrayList<Quotation> alQuotes = getQuotationsNewestVersions(user);
        ArrayList<Note> alNotes = getNotesAll(user);
        ArrayList<dbFile> alFiles = getFilesByUser(user);
        ArrayList<Person> alPersons = getPersonsNewest(user);

        ArrayList<Bibliography> alBibsAccept = new ArrayList<Bibliography>();
        ArrayList<Quotation> alQuotesAccept = new ArrayList<Quotation>();
        ArrayList<Note> alNotesAccept = new ArrayList<Note>();
        ArrayList<dbFile> alFilesAccept = new ArrayList<dbFile>();
        ArrayList<Person> alPersonsAccept = new ArrayList<Person>();


        for (int i = 0; i < alBibs.size(); i++) {
            int paramNumber = searchParameters.size();
            int paramNumberFound = 0; // for each succesfull encounter of the searchparamter, the numberFound is raised by one - if at the end, the numbers are equal, all parameters were found and the entry is not removed
            Iterator<String> iParams = searchParameters.iterator();
            String text = alBibs.get(i).getFieldSummary().toLowerCase();
            while (iParams.hasNext()) {
                String param = iParams.next().toLowerCase();
                // System.out.println("Looking for paramter " + param +" in "+text.replaceAll("[\\n\\r\\t]", "").trim());
                if (text.contains(param)) { // if text contains parameter, counter is adjusted
                    paramNumberFound++;
                    // System.out.println("found it in bibliography");
                }
            }
            // System.out.println("paramNumber "+paramNumber +" == paramNumberfound "+paramNumberFound +" = "+(paramNumber!=paramNumberFound));
            if (paramNumber == paramNumberFound) { // if number of found parameters matches total number of parameters, all parameters were found and entry is accepted as result
                alBibsAccept.add(alBibs.get(i));
            }
        }
        for (int i = 0; i < alQuotes.size(); i++) {
            int paramNumber = searchParameters.size();
            int paramNumberFound = 0; // for each succesfull encounter of the searchparamter, the numberFound is raised by one - if at the end, the numbers are equal, all parameters were found and the entry is not removed
            Iterator<String> iParams = searchParameters.iterator();
            String text = alQuotes.get(i).getFieldSummary().toLowerCase();
            while (iParams.hasNext()) {
                String param = iParams.next().toLowerCase();
                // System.out.println("Looking for parameter "+param);
                if (text.contains(param)) {
                    paramNumberFound++;
                    // System.out.println("found it in quote");
                }
            }
            // System.out.println("paramNumber "+paramNumber +" == paramNumberfound "+paramNumberFound +" = "+(paramNumber!=paramNumberFound));
            if (paramNumber == paramNumberFound) {
                alQuotesAccept.add(alQuotes.get(i));
            }
        }

        for (int i = 0; i < alNotes.size(); i++) {
            int paramNumber = searchParameters.size();
            int paramNumberFound = 0; // for each succesfull encounter of the searchparamter, the numberFound is raised by one - if at the end, the numbers are equal, all parameters were found and the entry is not removed
            Iterator<String> iParams = searchParameters.iterator();
            String text = alNotes.get(i).getFieldSummary().toLowerCase();
            while (iParams.hasNext()) {
                String param = iParams.next().toLowerCase();
                // System.out.println("Looking for parameter "+param);
                if (text.contains(param)) {
                    paramNumberFound++;
                    // System.out.println("found it in note");
                }
            }
            // System.out.println("paramNumber "+paramNumber +" == paramNumberfound "+paramNumberFound +" = "+(paramNumber!=paramNumberFound));
            if (paramNumber == paramNumberFound) {
                alNotesAccept.add(alNotes.get(i));
            }
        }

        for (int i = 0; i < alFiles.size(); i++) {
            int paramNumber = searchParameters.size();
            int paramNumberFound = 0; // for each succesfull encounter of the searchparamter, the numberFound is raised by one - if at the end, the numbers are equal, all parameters were found and the entry is not removed
            Iterator<String> iParams = searchParameters.iterator();
            String text = alFiles.get(i).getFieldSummary().toLowerCase();
            while (iParams.hasNext()) {
                String param = iParams.next().toLowerCase();
                // System.out.println("Looking for parameter "+param);
                if (text.contains(param)) {
                    paramNumberFound++;
                    // System.out.println("found it in note");
                }
            }

            // System.out.println("paramNumber "+paramNumber +" == paramNumberfound "+paramNumberFound +" = "+(paramNumber!=paramNumberFound));
            if (paramNumber == paramNumberFound) {
                alFilesAccept.add(alFiles.get(i));
            }
        }

        for (int i = 0; i < alPersons.size(); i++) {
            int paramNumber = searchParameters.size();
            int paramNumberFound = 0; // for each succesfull encounter of the searchparamter, the numberFound is raised by one - if at the end, the numbers are equal, all parameters were found and the entry is not removed
            Iterator<String> iParams = searchParameters.iterator();
            String text = alPersons.get(i).getFieldSummary().toLowerCase();
            while (iParams.hasNext()) {
                String param = iParams.next().toLowerCase();
                // System.out.println("Looking for parameter "+param);
                if (text.contains(param)) {
                    paramNumberFound++;
                    // System.out.println("found it in person");
                }
            }

            // System.out.println("paramNumber "+paramNumber +" == paramNumberfound "+paramNumberFound +" = "+(paramNumber!=paramNumberFound));
            if (paramNumber == paramNumberFound) {
                alPersonsAccept.add(alPersons.get(i));
            }
        }

        // System.out.println("db4o.returnSearchResultes() found "+alBibsAccept.size()+" Bibs and "+ alQuotesAccept.size()+" Quotes containing all search parameters. Closed database!");
        returnObject[0] = alBibsAccept;
        returnObject[1] = alQuotesAccept;
        returnObject[2] = alNotesAccept;
        returnObject[3] = alFilesAccept;
        returnObject[4] = alPersonsAccept;
        return returnObject;
    }

    /**
     * Returns user with given uuid - used e.g. for importing bibliographies to set existing user as owner, if uuid is found in database
     * @param userUUID  uuid of user to be getd
     * @return  user with given uuid
     */
    public User getUserByUUID(final UUID userUUID) {
        User user = null;

        // ObjectContainer db = Db4oEmbedded.openFile(Db4oEmbedded.newConfiguration(), "DBUser.db4o");
        List<User> result = userC.query(new Predicate<User>() {

            public boolean match(User user) {
                return user.getUUID().equals(userUUID);
            }
        });

        Iterator<User> iResult = result.iterator();
        while (iResult.hasNext()) {
            user = iResult.next();
        }
        return user;
    }

    /**
     *
     * @return  ArrayList<User> containing all users
     */
    public ArrayList<User> getUsers() {
        List<User> result = userC.query(new Predicate<User>() {

            public boolean match(User user) {
                return true;
            }
        });
        ArrayList<User> userList = new ArrayList<User>();
        for (User user : result) {
            userList.add(user);
        }
        return userList;
    }

    public void storePerson(Person person, User user) {
        // System.out.println("Storing person " + person.getUUID());
        db4o.objC.store(person);
        // System.out.println("Stored person");
    }

    public HashMap<String, Person> getPersonsByKey(final String key, final User user) {
        HashMap<String, Person> hmPersons = new HashMap<String, Person>();
        List<Person> result = null;
        // if a key was given, only get persons with key
        if (key != null && key.length() > 0) {
            result = db4o.objC.query(new Predicate<Person>() {

                public boolean match(Person search) {
                    return search.getKey().equals(key) & (!search.isPrivate() || user.hasAccessToPrivateObject(search.getUUID()));
                }
            });
            // if no key was given, return all keys
        } else {
            result = db4o.objC.query(new Predicate<Person>() {

                public boolean match(Person search) {
                    return !search.isPrivate() || user.hasAccessToPrivateObject(search.getUUID());
                }
            });
        }


        if (result.size() > 0) {
            // find the uuid families
            ArrayList<Person> alPersons = new ArrayList<Person>();
            alPersons.addAll(result);

            ArrayList<UUID> families = ObjectTools.filterFamiliesByUUID(ObjectTools.personToDBObject(alPersons));

            for (UUID uuid : families) {
                // System.out.println("Looking at family " + i++ + ":" + uuid);
                // find the newest within each family
                ArrayList<Person> family = new ArrayList<Person>();
                for (Person person : alPersons) {
                    if (person.getUUID().equals(uuid)) {
                        family.add(person);
                    }
                }
                // find the newest member of each family and add it to the hmPersons.HashMap
                int iNewest = ObjectTools.filterNewest(ObjectTools.personToDBObject(family));
                hmPersons.put(family.get(iNewest).getKey(), family.get(iNewest));
            }
        }
        return hmPersons;
    }

    public ArrayList<String> getAuthorKeyList(User user) {
        Object[] authorKeyListArray = getPersonsByKey("", user).keySet().toArray();
        ArrayList<String> authorKeyList = new ArrayList<String>();
        for (Object s : authorKeyListArray) {
            authorKeyList.add((String) s);
        }
        return authorKeyList;
    }

    public void storePersons(ArrayList<Person> alPersons, User user) {
        for (int i = 0; i < alPersons.size(); i++) {
            // System.out.println("Storing quote " + alBibliographies.get(i).getKey());
            storePerson(alPersons.get(i), user);
        }
    }

    /**
     * Get all notes for source object
     * @param user  current user
     * @param sourceUUID    uuid of source object
     * @param db    database ObjectContainer
     * @return  ArrayList<Note> containing all notes related to
     */
    public ArrayList<Note> getNotesBySourceUUIDNewest(final User user, final UUID sourceUUID) {
        List<Note> result = db4o.objC.query(new Predicate<Note>() {

            public boolean match(Note search) {
                return search.getRelatedObjects().contains(sourceUUID) & (!search.isPrivate() || (search.getUserUUID().equals(user.getUUID()) || user.hasAccessToPrivateObject(search.getUUID())));
            }
        });

        ArrayList<Note> alNotes = new ArrayList<Note>();
        alNotes.addAll(result);

        ArrayList<UUID> families = ObjectTools.filterFamiliesByUUID(ObjectTools.noteToDBObject(alNotes));
        ArrayList<Note> returnNotes = new ArrayList<Note>();

        for (UUID uuid : families) {
            // find the newest within each family
            ArrayList<Note> family = new ArrayList<Note>();
            for (Note note : alNotes) {
                if (note.getUUID().equals(uuid)) {
                    family.add(note);
                }
            }
            // find the newest member of each family and add it to the hmPersons.HashMap
            int iNewest = ObjectTools.filterNewest(ObjectTools.noteToDBObject(family));
            returnNotes.add(family.get(iNewest));
        }

        return returnNotes;
    }

    /**
     * Get all files for source object
     * @param user  current user
     * @param sourceUUID    uuid of source object
     * @param db    database ObjectContainer
     * @return  ArrayList<dbFile> containing all files related to sourceUUID
     */
    public ArrayList<dbFile> getFilesBySourceUUID(final UUID sourceUUID, final User user) {
        ArrayList<dbFile> alResult = new ArrayList<dbFile>();
        ArrayList<dbFile> filesAll = getFilesAll(user);
        for(dbFile file : filesAll){
            if(file.getRelatedObjects().contains(sourceUUID)){
                alResult.add(file);
            }
        }
        return alResult;
    }

    public ArrayList<dbFile> getFilesByUser(final User user) {
        List<dbFile> result = db4o.objC.query(new Predicate<dbFile>() {

            public boolean match(dbFile search) {
                return !search.isPrivate() || user.hasAccessToPrivateObject(search.getUUID());
            }
        });

        ArrayList<dbFile> alFiles = new ArrayList<dbFile>();
        alFiles.addAll(result);

        return alFiles;
    }

    public ArrayList<Note> getNotesByUserNewest(final User user) {
        List<Note> result = db4o.objC.query(new Predicate<Note>() {

            public boolean match(Note search) {
                return !search.isPrivate() || user.hasAccessToPrivateObject(search.getUUID());
            }
        });

        ArrayList<Note> alNotes = new ArrayList<Note>();
        alNotes.addAll(result);

        ArrayList<UUID> families = ObjectTools.filterFamiliesByUUID(ObjectTools.noteToDBObject(alNotes));
        ArrayList<Note> returnNotes = new ArrayList<Note>();

        for (UUID uuid : families) {
            // find the newest within each family
            ArrayList<Note> family = new ArrayList<Note>();
            for (Note note : alNotes) {
                if (note.getUUID().equals(uuid)) {
                    family.add(note);
                }
            }
            // find the newest member of each family and add it to the hmPersons.HashMap
            int iNewest = ObjectTools.filterNewest(ObjectTools.noteToDBObject(family));
            returnNotes.add(family.get(iNewest));
        }
        return returnNotes;
    }

    public ArrayList<dbFile> getFilesByUserNewest(final User user) {
        List<dbFile> result = db4o.objC.query(new Predicate<dbFile>() {

            public boolean match(dbFile search) {
                return !search.isPrivate() || user.hasAccessToPrivateObject(search.getUUID());
            }
        });

        ArrayList<dbFile> alFiles = new ArrayList<dbFile>();
        alFiles.addAll(result);

        ArrayList<UUID> families = ObjectTools.filterFamiliesByUUID(ObjectTools.fileToDBObject(alFiles));
        ArrayList<dbFile> returnFiles = new ArrayList<dbFile>();

        for (UUID uuid : families) {
            // find the newest within each family
            ArrayList<dbFile> family = new ArrayList<dbFile>();
            for (dbFile file : alFiles) {
                if (file.getUUID().equals(uuid)) {
                    family.add(file);
                }
            }
            // find the newest member of each family and add it to the hmPersons.HashMap
            int iNewest = ObjectTools.filterNewest(ObjectTools.fileToDBObject(family));
            returnFiles.add(family.get(iNewest));
        }

        return returnFiles;
    }

    public dbFile getFileByDateUUID(final User user, final UUID uuid, final Long revision) {

        List<dbFile> result = db4o.objC.query(new Predicate<dbFile>() {

            public boolean match(dbFile search) {
                return search.getUUID().equals(uuid) & search.getDateEdited().equals(revision) & (!search.isPrivate() || user.hasAccessToPrivateObject(search.getUUID()));
            }
        });

        if (result.size() > 0) {
            return result.get(0);
        }

        return null;
    }

    public void storeUser(User user) {
        // System.out.println("Stored user " + user.getName());
        // System.out.println("Has access to following uuids:");
        for (UUID uuid : user.getPrivateAccess().keySet()) {
            // System.out.println("   " + uuid);
        }
        // System.out.println("");

        // ObjectContainer db = Db4oEmbedded.openFile(Db4oEmbedded.newConfiguration(), "DBUser.db4o");
        user.setDateEdited(); // set the current date for the to be stored user element
        userC.store(user);
        // System.out.println("Stored user "+user.toString());
    }

    public ArrayList<Person> getPersonsNewest(final User user) {
        List<Person> result = db4o.objC.query(new Predicate<Person>() {

            public boolean match(Person person) {
                return !person.isPrivate() || user.hasAccessToPrivateObject(person.getUUID());
            }
        });
        ArrayList<Person> personList = new ArrayList<Person>();
        // add all found persons to list
        for (Person person : result) {
            // System.out.println("     *** db4o.retrievePersonsNewest() found person "+person.getName());
            personList.add(person);
        }

        personList = ObjectTools.dbObjectToPerson(ObjectTools.filterNewestByUUID(ObjectTools.personToDBObject(personList)));
        return personList;
    }

    /**
     * Erases all versions of person entry with uuid of given parameter person
     * @param person
     */
    public void erasePerson(final Person person) {
        List<Person> result = db4o.objC.query(new Predicate<Person>() {

            public boolean match(Person search) {
                return search.getUUID().equals(person.getUUID());
            }
        });

        for (int i = 0; i < result.size(); i++) {
            // System.out.println(i + ": Deleting person " + result.get(i).getName());
            db4o.objC.delete(result.get(i));
        }
    }

    public Note getNoteByUUID(final User user, final UUID uuid) {
        List<Note> result = db4o.objC.query(new Predicate<Note>() {

            public boolean match(Note search) {
                return search.getUUID().equals(uuid) & (!search.isPrivate() || (search.getUserUUID().equals(user.getUUID()) || user.hasAccessToPrivateObject(search.getUUID())));
            }
        });

        ArrayList<Note> alNotes = new ArrayList<Note>();
        alNotes.addAll(result);

        ArrayList<dbObject> objects = ObjectTools.noteToDBObject(alNotes);
        int filterNewest = ObjectTools.filterNewest(objects);

        if (filterNewest != -1) {
            return alNotes.get(filterNewest);
        } else {
            return null;
        }
    }

    public ArrayList<dbObject> getRevisionsByUUID(final User user, final UUID uuid) {      
        return this.getObjectsByUUID(uuid, user);
    }

    public dbFile getFileByUUID(final User user, final UUID uuid) {
        List<dbFile> result = db4o.objC.query(new Predicate<dbFile>() {

            public boolean match(dbFile search) {
                return search.getUUID().equals(uuid) & (!search.isPrivate() || (search.getUserUUID().equals(user.getUUID()) || user.hasAccessToPrivateObject(search.getUUID())));
            }
        });

        ArrayList<dbFile> al = new ArrayList<dbFile>();
        al.addAll(result);

        ArrayList<dbObject> objects = ObjectTools.fileToDBObject(al);
        int filterNewest = ObjectTools.filterNewest(objects);

        return al.get(filterNewest);
    }

    public ArrayList<Note> getNoteByDateUUID(final UUID uuid, final Long date, final User user) {
        List<Note> result = db4o.objC.query(new Predicate<Note>() {

            public boolean match(Note search) {
                return search.getUUID().equals(uuid) & search.getDateEdited().equals(date) & (!search.isPrivate() || user.hasAccessToPrivateObject(uuid));
            }
        });

        ArrayList<Note> vReturn = new ArrayList<Note>();
        for (Note note : result) {
            vReturn.add(note);
        }
        return vReturn;
    }

    public ArrayList<Person> getPersonByDateUUID(final UUID uuid, final Long date, final User user) {
        List<Person> result = db4o.objC.query(new Predicate<Person>() {

            public boolean match(Person search) {
                return search.getUUID().equals(uuid) & search.getDateEdited().equals(date) & (!search.isPrivate() || user.hasAccessToPrivateObject(uuid));
            }
        });

        ArrayList<Person> vReturn = new ArrayList<Person>();
        for (Person person : result) {
            vReturn.add(person);
        }
        return vReturn;
    }
}
