    /* The program is supposed to work as a bibliography library exporting and importing to bibTex format
Copyright (C) 2010  Georg Adelmann - mail@geoadel.net

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>. */
package net.geoadel.bibcite.DatabaseObjects;

import java.security.KeyPair;
import java.security.PrivateKey;
import java.security.PublicKey;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.UUID;
import javax.crypto.SealedObject;
import net.geoadel.bibcite.Cryptography.EncryptDecrypt;
import net.geoadel.bibcite.DataFields.SessionData;
import net.geoadel.bibcite.Database.dbModel;
import net.geoadel.bibcite.Database.db4o;
import org.apache.commons.lang.RandomStringUtils;
import org.jasypt.util.password.StrongPasswordEncryptor;

/**
 * User denotes a database object handling informations about users that enables them to log in, encrypt/decrypt private fields,
 * set a preferred language (multi language for interface not yet implemented) and also
 * different presets for preferences of the program (not yet implemented)
 *
 * @GPLStatus clear
 * @DocumentationStatus finished
 * @author Georg Adelmann
 */
public class User extends Person {

    private String defaultOperatingSystem;
    private String preferredLanguage; // the preferred language to be used e.g. for the user interface translation (not yet implemented)
    /**
     * The hash for the user login password, used to recognize the user
     */
    private String loginPasswordHash;
    /**
     * HashMap containing objects, the owner is granted access to, despite being private objects of other users (they need to allow access)
     * UUID - uuid of the private object
     * SealedObject - the decryption password of the object with the given uuid encrypted with the public key of this user
     */
    private HashMap<UUID, SealedObject> hmPrivateAccess;
    
    /**
     * The byte stream of the encrypted private key (encrypted using the user login password)
     * The unencrypted private key is never stored here, but only in the SessionData class
     */
    private byte[] keyPrivateBytesEncrypted;
    /**
     * The un-encrypted public key
     * Can be used by any user to encrypt data for this user only to be read
     */
    private PublicKey keyEncryptionPublic;
    // this hashmap stores for each operating system <stringm the file type <String, and its appointed program for opening it , String>
    private HashMap<String, HashMap<String, String>> hmProgramSettings = new HashMap<String, HashMap<String, String>>();

    /**
     * Constructor used by net.geoadel.bibcite.Database.db4o.java for constructing a new user, not yet existing in the database
     *
     * @param username  name
     * @param password  password
     * @param preferredLanguage preferred language
     */
    public User(String username, String password, String preferredLanguage) {
        setUUID(); // uuid is used for user identification within the program routines, once logged in

        SessionData.setLoginPassword(password);
        setLoginPasswordHash(password);

        // setDefaultOperatingSystem(getOperatingSystem());

        if (hmPrivateAccess == null) {
            hmPrivateAccess = new HashMap<UUID, SealedObject>();
        }

        setName(username);

        this.preferredLanguage = encrypt(preferredLanguage);

        if (keyEncryptionPublic == null) {
            KeyPair keys = new EncryptDecrypt().createKeyPair();
            /**
             * store the public key unencrypted
             */
            keyEncryptionPublic = keys.getPublic();

            /**
             * Store the private key as an encrypted byte array
             */
            PrivateKey privateKey = keys.getPrivate();
            keyPrivateBytesEncrypted = new EncryptDecrypt().encryptPrivateKey(privateKey);
        } else {
            // System.out.println("Old user with public key: " + getPublicKey());
        }

        setDefaultOperatingSystem(System.getProperty("os.name"));

        this.setDateEdited();
    }

    @Override
    public String toString() {
        String string = "";
        string = string + getName() + "\n";
        // string = string + getPreferredLanguage()+"\n";
        // string = string + getCurrentProgramSetting()+"\n";
        // string = string + hmProgramSettings+"\n";
        // string = string + dateEdited;
        return string;
    }

    /**
     * Get default user operating system , used for getting the settings for external programs, which should be used for
     * opening files
     * @return  name of operating system, preferably the suer set default operating system, if not available, return system variable
     */
    public String getDefaultOperatingSystem() {
        if (defaultOperatingSystem.isEmpty()) {
            return System.getProperty("os.name").toLowerCase();
        } else {
            return decrypt(defaultOperatingSystem).toLowerCase();
        }
    }

    public HashMap<String, String> getOperatingSystemSettings(String os){
        return hmProgramSettings.get(os.toLowerCase());
    }

    public boolean hasOperatingSystemSettings(String os){
        return hmProgramSettings.containsKey(os.toLowerCase());
    }

   /**
    * Set the default operating system settings to use for opening files with external programs
    * @param os
    */
    public void setDefaultOperatingSystem(String os) {
        this.defaultOperatingSystem = encrypt(os.toLowerCase());
    }

    /**
     * Set the default program for given type for given operating system
     * @param os        The operating system for which these settings are valid
     * @param type      The file type for which this program should be used
     * @param program   The program with which to open the file under the given operating system
     */
    public void setOperatingSystemDefaultProgram(String os, String type, String program){
        if(hmProgramSettings.containsKey(os.toLowerCase())){
            hmProgramSettings.get(os.toLowerCase()).put(type.toLowerCase(), program.toLowerCase());
        } else {
            HashMap<String, String> hm = new HashMap<String, String>();
            hm.put(type.toLowerCase(), program.toLowerCase());
            hmProgramSettings.put(os.toLowerCase(), hm);
        }
    }

    /**
     * Get the default program for given type and operating system, with which to open files of given type
     * @param os    Operating system settings to use
     * @param type  Type of file to open
     * @return      String containing the program execution command with which to open the file
     */
    public String getOperatingSystemDefaultProgram(String os, String type) {
        if (hmProgramSettings.containsKey(os.toLowerCase())) {
            if (hmProgramSettings.get(os.toLowerCase()).containsKey(type.toLowerCase())) {
                return hmProgramSettings.get(os).get(type.toLowerCase());
            } else {
                return null;
            }
        } else {
            return null;
        }
    }

    public String getPreferredLanguage() {
        return decrypt(preferredLanguage);
    }

    public void setPreferredLanguage(String language) {
        this.preferredLanguage = encrypt(language);
    }

    public String getLoginPasswordHash() {
        return loginPasswordHash;
    }

    /**
     * This needs to be called whenever the user password is changed, so that the permanent password can be decrypted with the old password and then be reencrypted with the new password.
     * @param password
     */
    private void setLoginPasswordHash(String password) {
        StrongPasswordEncryptor passwordEncryptor = new StrongPasswordEncryptor();
        this.loginPasswordHash = passwordEncryptor.encryptPassword(password);
    }

    /**
     *
     * @param password  the non encrypted password, the user enters at login/register panel
     * @return  boolean whether password hash matched the stored hash of the password used for registering the user first time
     */
    public boolean isUser(String password) {
        StrongPasswordEncryptor passwordEncryptor = new StrongPasswordEncryptor();
        return passwordEncryptor.checkPassword(password, this.loginPasswordHash);
    }

    /**
     * When the login password is changed, the permanent password needs to be reencrypted with the new login password
     * The passwordHash needs to be adjusted so that the new password is recognized at login by comparing the Hashes
     * @param newLoginPassword
     */
    public void changeLoginPassword(String newLoginPassword) {
        EncryptDecrypt crypt = new EncryptDecrypt();

        // change the private key encryption
        setLoginPasswordHash(newLoginPassword);

        PrivateKey privateKey = crypt.recreatePrivateKeyFromBytes(keyPrivateBytesEncrypted);

        // change the login password (needs to be done before setting the permanent password, because encryptPermanentPassword() gets the new user login password from the SessionData
        SessionData.setLoginPassword(newLoginPassword);

        // reencrypt privateKey and set bytes new
        keyPrivateBytesEncrypted = new EncryptDecrypt().encryptPrivateKey(privateKey);

        // store the user
        dbModel.storeUser(this);
    }

    public HashMap<String, String> getPreferencesPrograms() {
        throw new UnsupportedOperationException("Not yet implemented");
    }

    /**
     * Returns the public key of this user, so others can encrypt data that this user can encrypt
     * Used for encrypting objects shared by groups with help of a group password
     */
    public PublicKey getPublicKey() {
        return keyEncryptionPublic;
    }

    /**
     *
     * @return  private key used for decrypting sealed objects encrypted using this users public key
     */
    public PrivateKey getPrivateKey() {
        return new EncryptDecrypt().recreatePrivateKeyFromBytes(keyPrivateBytesEncrypted);
    }

    /**
     * @param objectUUID        uuid of the private object, this user is granted access to
     * @param passwordObject    password, with which the private object is encrypted
     */
    public void grantAccessToPrivateObject(UUID objectUUID, String passwordObject) {
        // System.out.println("Granting user "+getName()+" access to "+objectUUID);
        // use this users public key to encrypt the object password
        EncryptDecrypt crypt = new EncryptDecrypt();
        SealedObject encryptedPrivateObjectPassword = crypt.encryptPrivateObjectPassword(passwordObject, keyEncryptionPublic);
        // put object uuid and encrypted object password into HashMap
        hmPrivateAccess.put(objectUUID, encryptedPrivateObjectPassword);
        dbModel.storeUser(this);
    }

    /**
     *
     * @param uuid      uuid of object to be decrypted
     * @return          true, if password for object is in this users hmPrivateAccess
     */
    public boolean hasAccessToPrivateObject(UUID uuid) {
        return hmPrivateAccess.containsKey(uuid);
    }

    /**
     *
     * @param uuid
     * @return returns decrypted private  object password
     */
    public String getAccessPrivatePassword(UUID uuid) {
        // System.out.println("Checking user " + SessionData.getUser().getName() + " for access to private object " + uuid);
        /**
         * get password contained in SealedObject,
         * encrypted using the public key of user
         */
        SealedObject sealed = null;

        /*
         * If this object uuid does not have an entry yet, create one
         */
        if (!this.hmPrivateAccess.containsKey(uuid)) {
            String objectPassword = RandomStringUtils.randomAscii(30);
            SealedObject encryptPrivateObjectPassword = new EncryptDecrypt().encryptPrivateObjectPassword(objectPassword, keyEncryptionPublic);
            sealed = encryptPrivateObjectPassword;
            SessionData.getUser().hmPrivateAccess.put(uuid, sealed);
            // System.out.println("Password not found for " + uuid + " -> created new password");
            // store user with new passwords
            dbModel.storeUser(SessionData.getUser());
        } else {
            // System.out.println("Password found for " + uuid);
            sealed = SessionData.getUser().hmPrivateAccess.get(uuid);
        }

        /**
         * Retrieve the private key, to decrypt the private password
         */
        PrivateKey privateKey = new EncryptDecrypt().recreatePrivateKeyFromBytes(keyPrivateBytesEncrypted);

        String password = new EncryptDecrypt().decryptPrivateObjectPassword(sealed, privateKey);
        return password;
    }

    /**
     *
     * @return HashMap<UUID, SealedObject> containing the uuids and sealed passwords for objects this user has access to
     */
    public HashMap<UUID, SealedObject> getPrivateAccess() {
        return hmPrivateAccess;
    }

    /**
     * removeKeysUnused() cleans the user from all its access passwords to private objects which do not exist anymore.
     */
    public void removeKeysUnused() {
        if (SessionData.getUser() != null) {
            // check if private keys relate to objects in database - otherwise they are temporary keys for new editors and should be deleted as not to accumulate
            ArrayList<UUID> removeKeys = new ArrayList<UUID>();
            ArrayList<dbObject> alObjects = dbModel.getObjectsAll();
            ArrayList<UUID> allKeys = new ArrayList<UUID>();
            for (dbObject object : alObjects) {
                allKeys.add(object.getUUID());
            }

            for (UUID uuid : SessionData.getUser().getPrivateAccess().keySet()) {
                if (!allKeys.contains(uuid)) {
                    // since the keys cannot be removed from the list while iterating it [would result in ConcurrentModificationException],
                    // I store them in an arraylist for later removal
                    removeKeys.add(uuid);
                }
                // System.out.println("Key for object: "+uuid + " with password "+user.getAccessPrivatePassword(uuid));
            }
            for (UUID uuid : removeKeys) {
                SessionData.getUser().removeKeyUnused(uuid);
            }
        }

    }

    public void removeKeyUnused(UUID uuid) {
        SessionData.getUser().hmPrivateAccess.remove(uuid);
    }
}
