/* bibcite is a bibliography and citation library with exporting
and importing capabilities to bibTex format

Copyright (C) 2010  Georg Adelmann - mail@geoadel.net

This file is part of bibcite.

bibcite free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>. */


package net.geoadel.bibcite.DatabaseObjects;

import java.io.File;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.UUID;
import net.geoadel.bibcite.DataFields.SessionData;

/**
 * Database object to store file informations in database and file object for accessing real file on hard drive
 * (cannot be named just 'File' because that class already exists within java framework)
 * @GPLStatus clean
 * @DocumentationStatus finished
 * @author Georg Adelmann
 */
public final class dbFile extends dbObject {
    private String icon = "File16.png";
    // create dbFile specific data fields
    public File file;
    private Long size;
    private String wordList;
    
    /**
     * Instantiate a dbFile with all its fields set
     * @param file
     * @param size
     * @param user
     * @param uuid
     * @param sourceUUID
     * @param dateEdited
     * @param isPrivate
     * @param alRelatedObjects
     * @param alTags
     * @param hmCustomFields
     */
    public dbFile(File file, Long size, User user, UUID uuid, Long dateEdited, boolean isPrivate, ArrayList<UUID> alRelatedObjects, ArrayList<String> alTags, String originalName, String key, HashMap<String, String> hmCustomFields){
        this.isPrivate = isPrivate;

        setFile(file);
        setSize(size);

        setUserUUID(user.getUUID());
        setUUID(uuid);
        setDateEdited(dateEdited);
        this.setName(originalName);
        setRelatedObjects(alRelatedObjects);
        this.addTags(alTags);
        this.hmCustomFields = hmCustomFields;

        this.setKey(key);
    }

    /**
     * Instantiate a rudimentary dbFile, where many fields remain empty
     * @param file  file
     * @param user  user, that owns the file
     * @param sourceUUID    uuid of the related element, the file 'belongs' to
     * @param uuid  uuid of the
     * @param isPrivate whether the file is private or not
     * @param originalName  the original filename before it was moved into the database (needed to be able to restore the filename when moving it out of the database again)
     */
    public dbFile(File file, User user, ArrayList<UUID> alRelations, UUID uuid, boolean isPrivate, String originalName, String key){
        this.isPrivate = isPrivate;
        
        setFile(file);
        // setType(originalName.substring(originalName.lastIndexOf(".")+1).toUpperCase());
        setSize(Math.round(file.length()/1024.0d));

        setDateEdited();

        setUserUUID(user.getUUID());
        setUUID(uuid);
        this.setDateEdited();
        
        this.setName(originalName);
        setRelatedObjects(alRelations);
        this.alTags = new ArrayList<String>();
        this.hmCustomFields = new HashMap<String, String>();

        this.setKey(key);
    }

    /**
     * Used for temporary files to be imported into database
     * @param sourceUUID    uuid of related object
     */
    public dbFile(UUID sourceUUID){
        this.setName("");
        setUUID(UUID.randomUUID());
        setUserUUID(SessionData.getUser().getUUID());
        this.setDateEdited();
        this.addRelatedObject(sourceUUID);
        this.alTags = new ArrayList<String>();
        this.hmCustomFields = new HashMap<String, String>();
    }
    
    public String getIconFileName() {
        return icon;
    }
    
    /**
     * @return the file
     */
    public File getFile() {
        return file;
    }

    /**
     * @param file the file to set
     */
    public void setFile(File file) {
        this.file = file;
    }


    /**
     * @return the size
     */
    public Long getSize() {
        return size;
    }

    /**
     * @param size the size to set
     */
    public void setSize(Long size) {
        this.size = size;
    }

    /**
     * this toString() is used by the FileViewPanel.listFiles to display the fileNames but have access to the UUID as well
     * @return  file name
     */
    @Override
    public String toString(){
        return getName();
    }

    public String getRevision(){
        return getDateEdited().toString();
    }

    // used for searching for words; wordList should be filled by words of attached documents (implementation for pdf)
    public String getWordList(){
        return decrypt(wordList);
    }

    public void setWordList(String wordList){
        this.wordList = encrypt(wordList);
    }

    /**
     * String representation of most fields - used by db4o.java for searching for String parameters
     * @return  String containing fields relevant for search algorithm
     */
    public String getFieldSummary(){
        StringBuilder result = new StringBuilder();
        String NL = System.getProperty("line.separator");
        result.append(NL);
        result.append(getWordList()).append(NL);
        result.append(getName()).append(NL);
        return result.toString();
    }
}
