/* bibcite is a bibliography and citation library with exporting
and importing capabilities to bibTex format

Copyright (C) 2010  Georg Adelmann - mail@geoadel.net

This file is part of bibcite.

bibcite free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>. */

package net.geoadel.bibcite.DatabaseObjects;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.UUID;
import javax.swing.ImageIcon;
import net.geoadel.bibcite.Cryptography.EncryptDecrypt;
import net.geoadel.bibcite.DataFields.SessionData;

/**
 * the main database object which all others must extend for consistency and ease of database object access
 * Provides UUID, name, date of last edit, privacy status and other general fields
 * @GPLStatus clean
 * @DocumentationStatus finished
 * @author Georg Adelmann
 */
public class dbObject implements Comparable<dbObject> {

    /**
     * the uuid of the user 'owning' this object;
     * for users, the userUUID and the uuid are the same,
     * unless the user had been created by another user (not implemented)
     */
    static final String IMAGEPATH = "/net/geoadel/bibcite/Images/";
    private String icon ="file.png"; // default item
    private UUID userUUID;
    private String key = "";
    private String name = "";
    private String dateEdited = Long.toString(0L);
    boolean isPrivate = false;
    /**
     * Indicates, whether all fields should be encrypted all the time, not only private object fields would then be encrypted - WARNING: Very CPU intensive if set to true!
     */
    boolean alwaysEncrypt = false;
    
    private UUID uuid; // for users, the userUUID and the uuid are the same, see above
    ArrayList<UUID> alRelatedObjects = new ArrayList<UUID>();
    ArrayList<String> alTags = new ArrayList<String>();
    HashMap<String, String> hmCustomFields; // not yet accessed or accessible by UI

    /**
     * Only to be used for other elements implementing dbObject which set user themselves
     */
    public dbObject() {
    }

    /**
     * The standard
     */
    public dbObject(User user) {
        setUserUUID(user.getUUID());
    }

    /**
     * Should only be used for elements that are not to be stored (like root nodes in TreeViewBibliography.java)
     * @param name  the name of the dbObject
     */
    public dbObject(String name, User user) {
        this.setName(name);
        setUserUUID(SessionData.getUser().getUUID());
        setUUID(UUID.randomUUID());
    }

    /**
     *
     * @return
     */
    public UUID getUUID() {
        return uuid;
    }

    /**
     * sets a random UUID
     */
    public void setUUID() {
        uuid = UUID.randomUUID();
    }

    /**
     *
     * @param uuid uuid, the dbObject should have
     */
    public void setUUID(UUID uuid) {
        this.uuid = uuid;
    }

    public ImageIcon getIcon() {
        // icon is resolved dynamically
        return new ImageIcon(getClass().getResource(IMAGEPATH + getIconFileName()));
    }

    public void removeRelatedObject(UUID uuid) {
        if (this.alRelatedObjects.contains(uuid)) {
            alRelatedObjects.remove(uuid);
        }
    }

    public void addRelatedObject(UUID uuid) {
        if (!this.alRelatedObjects.contains(uuid)) {
            alRelatedObjects.add(uuid);
        }
    }

    public ArrayList<UUID> getRelatedObjects() {
        return alRelatedObjects;
    }

    public void setRelatedObjects(ArrayList<UUID> alRelatedObjects) {
        this.alRelatedObjects = alRelatedObjects;
    }

    public Long getDateEdited() {
        return Long.valueOf(dateEdited);
    }

    public void setDateEdited(Long dateEdited) {
        this.dateEdited = dateEdited.toString();
    }

    public Long setDateEdited() {
        SimpleDateFormat format = new SimpleDateFormat("yyyyMMddHHmmss");
        Date dt = new Date();
        this.dateEdited = Long.valueOf(format.format(dt)).toString();
        return Long.valueOf(this.dateEdited);
    }

    /**
     * compare the (encrypted) tag with the tags in the tag list. If the tag is not already in it, add it
     * @param tag the tag that should be added to the dbObject tags
     */
    public void addTag(String tag) {
        if (!this.alTags.contains(encrypt(tag))) {
            this.alTags.add(encrypt(tag));
        }
    }

    public void addTags(ArrayList<String> tags) {
        for (String tag : tags) {
            addTag(tag);
        }
    }

    /**
     * remove a tag after checking if it is in the tag list
     * @param tag   decrypted tag to be removed
     */
    public void removeTag(String tag) {
        // encrypt tag and see if its encrypted form is in alTags
        // if yes, remove it
        if (this.alTags.contains(encrypt(tag))) {
            this.alTags.remove(tag);
        }
    }

    /**
     * Get tags, decrypt if necessary
     * @return  ArrayList with decrypted tags
     */
    public ArrayList<String> getTags() {
        ArrayList<String> tempTags = new ArrayList<String>();
        for (int i = 0; i < alTags.size(); i++) {
            tempTags.add(decrypt(alTags.get(i)));
        }
        return tempTags;
    }

    public UUID getUserUUID() {
        return userUUID;
    }

    public void setUserUUID(UUID userUUID) {
        this.userUUID = userUUID;
    }

    /**
     *
     * @return
     */
    public String getName() {
        return decrypt(name);
    }

    /**
     * The object name
     * @param name
     */
    public void setName(String name) {
        this.name = encrypt(name);
    }

    /**
     * Returns the object key
     * @return  key
     */
    public String getKey() {
        return decrypt(key);
    }

    /**
     * Sets the object key
     * @param key object key
     */
    public void setKey(String key) {
        this.key = encrypt(key);
    }

    /**
     * Should always be set first, as to make sure, the other values are encrypted accordingly
     * @param isPrivate     boolean denoting, whether this dbObject is marked private
     */
    public void setIsPrivate(boolean isPrivate) {
        this.isPrivate = isPrivate;
    }

    public boolean isPrivate() {
        return isPrivate;
    }

    public HashMap<String, String> getCustomFields() {
        return hmCustomFields;
    }

    public void setCustomFields(HashMap<String, String> hmCustomFields) {
        this.hmCustomFields = hmCustomFields;
    }

    /**
     * returns String, that is encrypted, if isPrivate returns true
     *
     * @param text  the text
     * @return      the String, encrypted, if isPrivate is true
     */
    public String encrypt(String text) {
        // System.out.println("Encrypting text "+text+" with password "+isPrivate);
        net.geoadel.bibcite.Cryptography.EncryptDecrypt crypt = new net.geoadel.bibcite.Cryptography.EncryptDecrypt();
        if (isPrivate) {
            text = crypt.encryptPrivate(text, getUUID());
        } else {
            // if object is not private, use universal program password
            if(alwaysEncrypt){
                text = crypt.encryptPublic(text);
            }
        }
        // System.out.println("Encrypted: "+isPrivate+"; Text: "+text);
        return text;
    }

    /**
     * returns String, that is decrypted, if isPrivate returns true
     *
     * @param text  the text
     * @return      the String, decrypted, if isPrivate is true
     */
    public String decrypt(String text) {
        net.geoadel.bibcite.Cryptography.EncryptDecrypt crypt = new net.geoadel.bibcite.Cryptography.EncryptDecrypt();
        if (isPrivate) {
            // System.out.println("Decrypting private text: "+text);
            text = crypt.decryptPrivate(text, getUUID());
        } else {
            if(alwaysEncrypt){
                text = crypt.decryptPublic(text);
            }
        }
        return text;
    }

    public int compareTo(dbObject obj) {
        // COMPARE BY NAME
        // Calculate leading characters' numerical difference
        int numericalDiff = 0;
        try {
            numericalDiff = Integer.valueOf(getName().replaceAll("\\W", "").split("\\D", 2)[0]) - Integer.valueOf(obj.getName().replaceAll("\\W", "").split("\\D", 2)[0]);
        } catch (NumberFormatException e) { }
            // return String compateTo if neither name starts with a digit
        return numericalDiff != 0 ? numericalDiff : getName().replaceAll("\\W", "").compareToIgnoreCase(obj.getName().replaceAll("\\W", ""));
    }

    public String getIconFileName() {
        return icon;
    }
}
