/* bibcite is a bibliography and citation library with exporting
and importing capabilities to bibTex format
Copyright (C) 2010  Georg Adelmann - mail@geoadel.net

This file is part of bibcite.

bibcite free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>. */

/*
 * BibliographyEditor.java
 *
 * Created on 31.05.2010, 23:06:09
 */
package net.geoadel.bibcite.GUI.ObjectEditors;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import net.geoadel.bibcite.GUI.ObjectListPanels.FileEditorPanel;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;

import javax.swing.DefaultComboBoxModel;

import net.geoadel.bibcite.DocType.*;
import net.geoadel.bibcite.Database.dbModel;
import net.geoadel.bibcite.Tools.BaseN;
import net.geoadel.bibcite.DatabaseObjects.*;
import java.awt.image.BufferedImage;
import java.io.File;
import java.util.Collections;
import javax.swing.DefaultListModel;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import net.geoadel.bibcite.DataFields.Languages;
import net.geoadel.bibcite.DataFields.SessionData;
import net.geoadel.bibcite.GUI.AttributeEditors.AccessManagerGUI;
import net.geoadel.bibcite.GUI.ImageViewerPanel;
import net.geoadel.bibcite.GUI.ObjectListPanels.QuotationEditorPanel;
import net.geoadel.bibcite.GUI.AttributeEditors.RelationManager;
import net.geoadel.bibcite.GUI.ObjectListPanels.NoteEditorPanel;
import net.geoadel.bibcite.GUI.ObjectListPanels.TextAreaPanel;
import net.geoadel.bibcite.ImportExport.Import_bibTexByISBNDOI;
import net.geoadel.bibcite.Tools.StringTools;

/**
 * Provides a JPanel for displaying and editing Bibliography data. Incorporates QuotationViewer to display related quotes
 * 
 * @GPLStatus clean
 * @DocumentationStatus finished
 * @author Georg Adelmann
 */
public class BibliographyEditor extends javax.swing.JPanel {

    volatile boolean loading = true;  // used for cbRevision to not go into a revision loading loop because of ssetting cbRevision to current revision (then loading the selected revision and selecting that revision which then would load the selected revision....)
    boolean isPrivate = false; // this is used, to check, whether text decryption is needed by decrypt(String text). This way, the syntax is shortened
    ArrayList<String> alLanguageList = new ArrayList<String>(Arrays.asList("German", "English", "Spanish", "French"));

    private long dateEdited;
    private String firstAuthorLastName = ""; // useful for bib key generation
    private ArrayList<String> bibKeys; // ArrayList of all bibliography keys in the database
    // true means, the key will not be adapted to a change of year or author-names
    // components needed for file and image handling
    BufferedImage bImage;
    File selectedFile;
    String fileSeparator = System.getProperty("file.separator");
    ImageViewerPanel iv;
    DefaultListModel dlm = new DefaultListModel();
    public Bibliography bib;
    private AccessManagerGUI accessManager;
    ArrayList<Quotation> alQuotes;

    /**
     * Load a bibliography entry into the editor
     *
     * @param user  current user
     * @param bib   bibliography to be loaded
     * @param family        all bibliography entries with same uuid as bib, to get access on other revisions
     */
    public BibliographyEditor(Bibliography bib) {
        this.bib = bib;

        accessManager = new AccessManagerGUI(bib);
        initComponents();
        initComponentDefaultState();
        initData(bib);
        loading = false;
    }

    /**
     * sets the data for all the GUI elements
     * @param bibLoad   the bib entry to be displayed
     * @param family    its uuid family (other revisions)
     * @param user      the user, necessary to decrypt the string data if private
     */
    private void initData(Bibliography bibLoad) {

        this.bib.setUUID(bibLoad.getUUID()); // set the uuid field, so it can be retrieved for example by changing revisions
        if (bibLoad == null) {
            this.bib.setUUID();
        } else {
            this.bib.setUUID(bibLoad.getUUID());
        }

        this.bib = bibLoad;

        /**
         * if the key field is empty (can happen e.g. importing JabRef, because JabRef does not always automatically assign a key, let the program calculate the key
         */
        if (bibLoad.getKey() == null) {
            bibLoad.setKey("");
        }

        if (bibLoad.getKey().isEmpty()) {
            cbKeyFixed.setSelected(false);
        } else {
            cbKeyFixed.setSelected(true);
        }
        taAbstract.setText(bibLoad.getBibAbstract());
        taAddress.setText(bibLoad.getAddress());
        System.out.println("Loaded bib with Booktitle: " + bibLoad.getBooktitle());
        tfBooktitle.setText(bibLoad.getBooktitle());
        System.out.println("tfBooktitle is set to: " + tfBooktitle.getText());
        cbCrossRef.setSelectedItem(bibLoad.getCrossref());
        ArrayList<String> keys = dbModel.getObjectKeys();
        // add an empty key as first entry to avoid automatically having a selected cross-reference replace fields in bibliography
        keys.add(0, "");
        cbCrossRef.setModel(new DefaultComboBoxModel(keys.toArray()));

        tfHowPublished.setText(bibLoad.getHowpublished());
        tfInstitution.setText(bibLoad.getInstitution());
        tfJournal.setText(bibLoad.getJournal());
        // System.out.println("imported journal name >> " + tfJournal.getText());
        tfJournal.updateUI();
        tfKeyValue.setText(bibLoad.getKey());
        tfOrganisation.setText(bibLoad.getOrganization());
        tfPages.setText(bibLoad.getPages());
        tfPublisher.setText(bibLoad.getPublisher());
        tfSchool.setText(bibLoad.getSchool());
        tfSeries.setText(bibLoad.getSeries());

        tfTitle.setText(bibLoad.getTitle());
        tfSubtitle.setText(bibLoad.getSubtitle());

        cbType.setSelectedItem(bibLoad.getType());

        if (!alLanguageList.contains(bibLoad.getLanguage())) {
            alLanguageList.add(bibLoad.getLanguage());

        }
        cbLanguage.setModel(new DefaultComboBoxModel(alLanguageList.toArray()));

        cbLanguage.setSelectedItem(bibLoad.getLanguage());
        cbLanguage.updateUI();
        cbLanguage.getSelectedItem();

        tfChapter.setText(String.valueOf(bibLoad.getChapter()));
        tfEdition.setText(String.valueOf(bibLoad.getEdition()));
        tfMonth.setText(String.valueOf(bibLoad.getMonth()));
        tfYear.setText(String.valueOf(bibLoad.getYear()));
        tfNumber.setText(String.valueOf(bibLoad.getNumber()));
        tfVolume.setText(String.valueOf(bibLoad.getVolume()));

        taAuthors.setText(bibLoad.getAuthorNameAsText());
        taEditors.setText(bibLoad.getEditorNameAsText());

        cbPrivate.setSelected(bibLoad.isPrivate());

        tfLicense.setText(bibLoad.getLicense());

        tfURL.setText(bibLoad.getUrl());
        tfURLDate.setText(bibLoad.getUrlDate() + "");

        // this.tfTy.setText(bibLoad.getTypeTechReport());
        tfEPrint.setText(bibLoad.getEPrint());

        tfIdentifier.setText(bibLoad.getIdentifier());

        ArrayList<String> tags = bibLoad.getTags();
        String sTags = "";
        // System.out.println("Sorting through quote tags:");
        for (int i = 0; i < tags.size(); i++) {
            // System.out.println(tags.get(i));
            sTags = sTags.concat(tags.get(i));
            if (i < tags.size() - 1) {
                sTags = sTags.concat(", ");
            }
        }
        tfTags.setText(sTags);
        System.out.println("Loaded revision " + bibLoad.getDateEdited());
    }

    public void saveBibliography() {
        SimpleDateFormat format = new SimpleDateFormat("yyyyMMddHHmmss");
        Date dt = new Date();
        dateEdited = Long.valueOf(format.format(dt));

        String title = tfTitle.getText();
        String subtitle = tfSubtitle.getText();
        String booktitle = tfBooktitle.getText();

        long urlDate = 0l;
        try {
            urlDate = Long.valueOf(tfURLDate.getText());
        } catch (NumberFormatException nfe) {
            // nfe.printStackTrace();
        }
        String language = (String) cbLanguage.getSelectedItem();
        // System.out.println(language);

        Bibliography bibStore = null;

        // System.out.println("Abstract " + taAbstract.getText());

        String[] tags = tfTags.getText().split(", ");
        ArrayList<String> alTags = new ArrayList<String>();
        alTags.addAll(Arrays.asList(tags));
        System.out.println("Retrieving booktitle from textfield: " + tfBooktitle.getText());
        bib = new Bibliography(bib.getUUID(), taAbstract.getText(), taAddress.getText(), tfBooktitle.getText(), String.valueOf(cbCrossRef.getSelectedItem()), tfHowPublished.getText(), tfInstitution.getText(), tfJournal.getText(), tfKeyValue.getText(), tfOrganisation.getText(), tfPublisher.getText(), tfSchool.getText(), tfSeries.getText(), title, String.valueOf(cbType.getSelectedItem()), tfChapter.getText(), tfEdition.getText(), tfMonth.getText(), tfYear.getText(), tfNumber.getText(), tfPages.getText(), tfVolume.getText(), StringTools.parseNamesTextArea(taAuthors.getText()), StringTools.parseNamesTextArea(taEditors.getText()), tfLicense.getText(), tfURL.getText(), "", tfEPrint.getText(), urlDate, SessionData.getUser(), subtitle, dateEdited, cbPrivate.isSelected(), language, tfIdentifier.getText(), alTags);
        bib.setName(title);
        System.out.println("Sending book to dbModel with booktitle: " + bib.getBooktitle());
        dbModel.storeObject(bib);

    }

    /**
     * initiates the default states and configurations for the GUI elements
     */
    private void initComponentDefaultState() {
        // apply the document format to the number fields so they only accept digits (different maximum lenght)
        tfYear.setDocument(new DocumentTypeFourDigit());
        tfMonth.setDocument(new DocumentTypeTwoDigit());
        tfChapter.setDocument(new DocumentTypeSixDigit());
        tfVolume.setDocument(new DocumentTypeSixDigit());
        tfEdition.setDocument(new DocumentTypeSixDigit());

        tfURLDate.setDocument(new DocumentTypeEightDigit());

        cbType.setSelectedItem("book");
        QuotationEditorPanel quotePanel = new QuotationEditorPanel(this, bib.getUUID());
        // System.out.println("BibliographyEditor.initComponentDefaultState() calls new FileViewPanel using bib uuid " + uuid);
        jTabbedPane1.add(quotePanel, "Quotes"); // add the QuotationEditor Panel
        jTabbedPane1.add(new FileEditorPanel(bib.getUUID()), "Files"); // add the FileView Panel to the Tabbed pane
        jTabbedPane1.add(new NoteEditorPanel(bib.getUUID()), "Notes"); // add the NoteEditor Panel

        if (bib != null && (!bib.getTitle().isEmpty() || !bib.getYear().isEmpty())) {
            jTabbedPane1.add(new BibliographyViewer(bib, dbModel.getBibliographyQuotes(bib)), "Simple Viewer");
            // set last component of jTabbedPane1 as currently active, so it is displayed... should be BibliographyViewer

            jTabbedPane1.setSelectedIndex(jTabbedPane1.getComponentCount() - 1);
        }
        // initialize the bibKeys to their current value - used in updateKeyValue() to ensure that no two keys are the same

        jTabbedPane1.add(new RelationManager(bib), "Relations");

        bibKeys = dbModel.getObjectKeys();
        // System.out.println("initComponentDefaultState() is finished and does not call another method");

        ArrayList<String> languages = Languages.getLanguages();
        if (!languages.contains(SessionData.getUser().getPreferredLanguage())) {
            languages.add(SessionData.getUser().getPreferredLanguage());
        }
        if (!languages.contains(bib.getLanguage())) {
            languages.add(bib.getLanguage());
        }
        Collections.sort(languages);

        cbLanguage.setModel(new DefaultComboBoxModel(languages.toArray()));
        cbLanguage.setEditable(true);
        if (bib.getLanguage() != null & !bib.getLanguage().isEmpty()) {
            cbLanguage.setSelectedItem(bib.getLanguage());
        } else {
            cbLanguage.setSelectedItem(SessionData.getUser().getPreferredLanguage());
        }

        final ComboBoxRevisionAccess cbRevisions = new ComboBoxRevisionAccess(dbModel.getRevisionsOf(bib.getUUID()));
        cbRevisions.getModel().setSelectedItem(bib);
        cbRevisions.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                bib = (Bibliography)cbRevisions.getSelectedItem();
                initData((Bibliography) cbRevisions.getSelectedItem());
            }
        });
        jToolBar1.add(cbRevisions);
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        jTabbedPane1 = new javax.swing.JTabbedPane();
        jScrollPane1 = new javax.swing.JScrollPane();
        panelGeneral = new javax.swing.JPanel();
        panelGeneral_type_key_rev = new javax.swing.JPanel();
        labelType = new javax.swing.JLabel();
        cbType = new javax.swing.JComboBox(net.geoadel.bibcite.DataFields.BibliographyTypes.getBibTypes());
        labelKey = new javax.swing.JLabel();
        cbKeyFixed = new javax.swing.JCheckBox();
        tfKeyValue = new javax.swing.JTextField();
        panelGeneral_titles = new javax.swing.JPanel();
        labelTitle = new javax.swing.JLabel();
        labelSubtitle = new javax.swing.JLabel();
        labelBooktitle = new javax.swing.JLabel();
        tfTitle = new javax.swing.JTextField();
        tfSubtitle = new javax.swing.JTextField();
        tfBooktitle = new javax.swing.JTextField();
        labelSeries = new javax.swing.JLabel();
        labelJournal = new javax.swing.JLabel();
        tfJournal = new javax.swing.JTextField();
        tfSeries = new javax.swing.JTextField();
        panelAuthorEditorPublisher = new javax.swing.JPanel();
        labelAuthors = new javax.swing.JLabel();
        labelEditors = new javax.swing.JLabel();
        jScrollPane4 = new javax.swing.JScrollPane();
        taAuthors = new javax.swing.JTextArea();
        jScrollPane5 = new javax.swing.JScrollPane();
        taEditors = new javax.swing.JTextArea();
        tfPublisher = new javax.swing.JTextField();
        labelPublisher = new javax.swing.JLabel();
        jLabel4 = new javax.swing.JLabel();
        jScrollPane6 = new javax.swing.JScrollPane();
        taAddress = new javax.swing.JTextArea();
        panelYearOtherNumbers = new javax.swing.JPanel();
        labelYear = new javax.swing.JLabel();
        tfYear = new javax.swing.JTextField();
        labelMonth = new javax.swing.JLabel();
        tfMonth = new javax.swing.JTextField();
        labelVolume = new javax.swing.JLabel();
        tfVolume = new javax.swing.JTextField();
        labelNumber = new javax.swing.JLabel();
        tfNumber = new javax.swing.JTextField();
        labelPages = new javax.swing.JLabel();
        tfPages = new javax.swing.JTextField();
        labelChapter = new javax.swing.JLabel();
        tfChapter = new javax.swing.JTextField();
        jLabel11 = new javax.swing.JLabel();
        tfEdition = new javax.swing.JTextField();
        panelLicense_Organisations = new javax.swing.JPanel();
        labelOrganisation = new javax.swing.JLabel();
        labelInstitution = new javax.swing.JLabel();
        tfInstitution = new javax.swing.JTextField();
        tfOrganisation = new javax.swing.JTextField();
        labelSchool = new javax.swing.JLabel();
        tfSchool = new javax.swing.JTextField();
        labelHowPublished = new javax.swing.JLabel();
        tfHowPublished = new javax.swing.JTextField();
        labelEPrint = new javax.swing.JLabel();
        tfEPrint = new javax.swing.JTextField();
        labelCrossRef = new javax.swing.JLabel();
        labelLicense = new javax.swing.JLabel();
        cbCrossRef = new JComboBox();
        tfLicense = new javax.swing.JTextField();
        lLanguage = new javax.swing.JLabel();
        cbLanguage = new javax.swing.JComboBox();
        jScrollPane2 = new javax.swing.JScrollPane();
        panelContent = new javax.swing.JPanel();
        jPanel3 = new javax.swing.JPanel();
        labelURL = new javax.swing.JLabel();
        tfURL = new javax.swing.JTextField();
        labelURLDate = new javax.swing.JLabel();
        tfURLDate = new javax.swing.JTextField();
        jLabel2 = new javax.swing.JLabel();
        tfIdentifier = new javax.swing.JTextField();
        bIdentifierImport = new javax.swing.JButton();
        jPanel5 = new javax.swing.JPanel();
        labelAbstract = new javax.swing.JLabel();
        jScrollPane7 = new javax.swing.JScrollPane();
        taAbstract = new javax.swing.JTextArea();
        jLabel5 = new javax.swing.JLabel();
        tfTags = new javax.swing.JTextField();
        jToolBar1 = new javax.swing.JToolBar();
        lTitle = new javax.swing.JLabel();
        cbPrivate = new javax.swing.JCheckBox();
        bPrivate = new javax.swing.JButton();
        bSave = new javax.swing.JButton();
        bDelete = new javax.swing.JButton();
        jLabel3 = new javax.swing.JLabel();
        labelRevision = new javax.swing.JLabel();

        jTabbedPane1.setMinimumSize(new java.awt.Dimension(72, 400));
        jTabbedPane1.setPreferredSize(new java.awt.Dimension(112, 680));

        jScrollPane1.setMinimumSize(new java.awt.Dimension(612, 662));
        jScrollPane1.setPreferredSize(new java.awt.Dimension(612, 662));

        panelGeneral.setMinimumSize(new java.awt.Dimension(600, 700));
        panelGeneral.setPreferredSize(new java.awt.Dimension(600, 700));

        labelType.setText("Type");

        cbType.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                cbTypeActionPerformed(evt);
            }
        });

        labelKey.setText("Key");

        cbKeyFixed.setToolTipText("If enabled, a lock is in place, preventing changes to the key by either changes to author or year. Only change this status manually if you are certain you want it to be changed.");
        cbKeyFixed.setPressedIcon(new javax.swing.ImageIcon(getClass().getResource("/net/geoadel/bibcite/Images/lock.png"))); // NOI18N
        cbKeyFixed.addChangeListener(new javax.swing.event.ChangeListener() {
            public void stateChanged(javax.swing.event.ChangeEvent evt) {
                cbKeyFixedStateChanged(evt);
            }
        });

        javax.swing.GroupLayout panelGeneral_type_key_revLayout = new javax.swing.GroupLayout(panelGeneral_type_key_rev);
        panelGeneral_type_key_rev.setLayout(panelGeneral_type_key_revLayout);
        panelGeneral_type_key_revLayout.setHorizontalGroup(
            panelGeneral_type_key_revLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(panelGeneral_type_key_revLayout.createSequentialGroup()
                .addContainerGap()
                .addComponent(labelType)
                .addGap(41, 41, 41)
                .addComponent(cbType, javax.swing.GroupLayout.PREFERRED_SIZE, 152, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(labelKey, javax.swing.GroupLayout.PREFERRED_SIZE, 35, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(tfKeyValue, javax.swing.GroupLayout.PREFERRED_SIZE, 217, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addComponent(cbKeyFixed)
                .addContainerGap(60, Short.MAX_VALUE))
        );
        panelGeneral_type_key_revLayout.setVerticalGroup(
            panelGeneral_type_key_revLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, panelGeneral_type_key_revLayout.createSequentialGroup()
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .addGroup(panelGeneral_type_key_revLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(cbKeyFixed)
                    .addGroup(panelGeneral_type_key_revLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                        .addComponent(labelType)
                        .addComponent(cbType, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addComponent(labelKey)
                        .addComponent(tfKeyValue, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)))
                .addGap(34, 34, 34))
        );

        panelGeneral_titles.setPreferredSize(new java.awt.Dimension(500, 154));

        labelTitle.setText("Title");

        labelSubtitle.setText("Subtitle");

        labelBooktitle.setText("Booktitle");

        tfTitle.addCaretListener(new javax.swing.event.CaretListener() {
            public void caretUpdate(javax.swing.event.CaretEvent evt) {
                tfTitleCaretUpdate(evt);
            }
        });

        labelSeries.setText("Series");

        labelJournal.setText("Journal");

        javax.swing.GroupLayout panelGeneral_titlesLayout = new javax.swing.GroupLayout(panelGeneral_titles);
        panelGeneral_titles.setLayout(panelGeneral_titlesLayout);
        panelGeneral_titlesLayout.setHorizontalGroup(
            panelGeneral_titlesLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(panelGeneral_titlesLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(panelGeneral_titlesLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(labelTitle)
                    .addComponent(labelSubtitle)
                    .addComponent(labelBooktitle)
                    .addComponent(labelJournal)
                    .addComponent(labelSeries))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(panelGeneral_titlesLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(tfSubtitle, javax.swing.GroupLayout.DEFAULT_SIZE, 515, Short.MAX_VALUE)
                    .addComponent(tfTitle, javax.swing.GroupLayout.DEFAULT_SIZE, 515, Short.MAX_VALUE)
                    .addComponent(tfBooktitle, javax.swing.GroupLayout.DEFAULT_SIZE, 515, Short.MAX_VALUE)
                    .addComponent(tfJournal, javax.swing.GroupLayout.DEFAULT_SIZE, 515, Short.MAX_VALUE)
                    .addComponent(tfSeries, javax.swing.GroupLayout.DEFAULT_SIZE, 515, Short.MAX_VALUE))
                .addContainerGap())
        );
        panelGeneral_titlesLayout.setVerticalGroup(
            panelGeneral_titlesLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(panelGeneral_titlesLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(panelGeneral_titlesLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(labelTitle)
                    .addComponent(tfTitle, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(panelGeneral_titlesLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(labelSubtitle)
                    .addComponent(tfSubtitle, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(panelGeneral_titlesLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(labelBooktitle)
                    .addComponent(tfBooktitle, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(panelGeneral_titlesLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(labelJournal)
                    .addComponent(tfJournal, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(panelGeneral_titlesLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(labelSeries)
                    .addComponent(tfSeries, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addContainerGap(33, Short.MAX_VALUE))
        );

        panelAuthorEditorPublisher.setMinimumSize(new java.awt.Dimension(400, 192));

        labelAuthors.setText("Authors");

        labelEditors.setText("Editors");

        jScrollPane4.setPreferredSize(new java.awt.Dimension(152, 72));

        taAuthors.setColumns(20);
        taAuthors.setToolTipText("Surname and first name, separated by comma; new line for each name");
        taAuthors.setDragEnabled(true);
        taAuthors.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                taAuthorsMouseClicked(evt);
            }
        });
        taAuthors.addCaretListener(new javax.swing.event.CaretListener() {
            public void caretUpdate(javax.swing.event.CaretEvent evt) {
                taAuthorsCaretUpdate(evt);
            }
        });
        jScrollPane4.setViewportView(taAuthors);

        jScrollPane5.setPreferredSize(new java.awt.Dimension(150, 20));

        taEditors.setColumns(20);
        taEditors.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                taEditorsMouseClicked(evt);
            }
        });
        jScrollPane5.setViewportView(taEditors);

        labelPublisher.setText("Publisher");

        jLabel4.setText("Address");

        taAddress.setColumns(20);
        taAddress.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                taAddressMouseClicked(evt);
            }
        });
        jScrollPane6.setViewportView(taAddress);

        javax.swing.GroupLayout panelAuthorEditorPublisherLayout = new javax.swing.GroupLayout(panelAuthorEditorPublisher);
        panelAuthorEditorPublisher.setLayout(panelAuthorEditorPublisherLayout);
        panelAuthorEditorPublisherLayout.setHorizontalGroup(
            panelAuthorEditorPublisherLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(panelAuthorEditorPublisherLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(panelAuthorEditorPublisherLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(labelAuthors)
                    .addComponent(labelEditors)
                    .addComponent(labelPublisher)
                    .addComponent(jLabel4))
                .addGap(16, 16, 16)
                .addGroup(panelAuthorEditorPublisherLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                    .addComponent(jScrollPane6, javax.swing.GroupLayout.DEFAULT_SIZE, 507, Short.MAX_VALUE)
                    .addComponent(tfPublisher, javax.swing.GroupLayout.Alignment.LEADING, javax.swing.GroupLayout.DEFAULT_SIZE, 507, Short.MAX_VALUE)
                    .addComponent(jScrollPane5, javax.swing.GroupLayout.DEFAULT_SIZE, 507, Short.MAX_VALUE)
                    .addComponent(jScrollPane4, javax.swing.GroupLayout.Alignment.LEADING, javax.swing.GroupLayout.DEFAULT_SIZE, 507, Short.MAX_VALUE))
                .addContainerGap())
        );
        panelAuthorEditorPublisherLayout.setVerticalGroup(
            panelAuthorEditorPublisherLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(panelAuthorEditorPublisherLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(panelAuthorEditorPublisherLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(labelAuthors)
                    .addComponent(jScrollPane4, javax.swing.GroupLayout.PREFERRED_SIZE, 41, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(panelAuthorEditorPublisherLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                    .addComponent(labelEditors)
                    .addComponent(jScrollPane5, javax.swing.GroupLayout.PREFERRED_SIZE, 40, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(panelAuthorEditorPublisherLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(tfPublisher, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(labelPublisher))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(panelAuthorEditorPublisherLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(jLabel4)
                    .addComponent(jScrollPane6, javax.swing.GroupLayout.DEFAULT_SIZE, 41, Short.MAX_VALUE))
                .addContainerGap())
        );

        labelYear.setText("Year");

        tfYear.addCaretListener(new javax.swing.event.CaretListener() {
            public void caretUpdate(javax.swing.event.CaretEvent evt) {
                tfYearCaretUpdate(evt);
            }
        });

        labelMonth.setText("Month");

        labelVolume.setText("Volume");

        labelNumber.setText("Number");

        tfNumber.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                tfNumberActionPerformed(evt);
            }
        });

        labelPages.setText("Pages");

        labelChapter.setText("Chapter");

        tfChapter.setMinimumSize(new java.awt.Dimension(30, 24));

        jLabel11.setText("Edition");

        tfEdition.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                tfEditionActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout panelYearOtherNumbersLayout = new javax.swing.GroupLayout(panelYearOtherNumbers);
        panelYearOtherNumbers.setLayout(panelYearOtherNumbersLayout);
        panelYearOtherNumbersLayout.setHorizontalGroup(
            panelYearOtherNumbersLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(panelYearOtherNumbersLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(panelYearOtherNumbersLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(labelVolume)
                    .addComponent(labelYear))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(panelYearOtherNumbersLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                    .addComponent(tfVolume)
                    .addComponent(tfYear, javax.swing.GroupLayout.DEFAULT_SIZE, 55, Short.MAX_VALUE))
                .addGap(24, 24, 24)
                .addGroup(panelYearOtherNumbersLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(jLabel11)
                    .addComponent(labelMonth))
                .addGap(23, 23, 23)
                .addGroup(panelYearOtherNumbersLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                    .addComponent(tfEdition)
                    .addComponent(tfMonth, javax.swing.GroupLayout.PREFERRED_SIZE, 47, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addGap(18, 18, 18)
                .addGroup(panelYearOtherNumbersLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(labelPages)
                    .addComponent(labelNumber))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(panelYearOtherNumbersLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                    .addComponent(tfNumber)
                    .addComponent(tfPages, javax.swing.GroupLayout.DEFAULT_SIZE, 96, Short.MAX_VALUE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(labelChapter)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(tfChapter, javax.swing.GroupLayout.PREFERRED_SIZE, 52, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addContainerGap())
        );
        panelYearOtherNumbersLayout.setVerticalGroup(
            panelYearOtherNumbersLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(panelYearOtherNumbersLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(panelYearOtherNumbersLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, panelYearOtherNumbersLayout.createSequentialGroup()
                        .addComponent(labelMonth)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                        .addGroup(panelYearOtherNumbersLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                            .addComponent(tfEdition, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                            .addComponent(labelPages)
                            .addComponent(tfPages, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                            .addComponent(labelChapter)
                            .addComponent(tfChapter, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                            .addComponent(jLabel11)))
                    .addGroup(panelYearOtherNumbersLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                        .addComponent(tfMonth, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addComponent(labelNumber)
                        .addComponent(tfNumber, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                    .addGroup(panelYearOtherNumbersLayout.createSequentialGroup()
                        .addGroup(panelYearOtherNumbersLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                            .addComponent(labelYear)
                            .addComponent(tfYear, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, 29, Short.MAX_VALUE)
                        .addGroup(panelYearOtherNumbersLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                            .addComponent(labelVolume)
                            .addComponent(tfVolume, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))))
                .addContainerGap())
        );

        labelOrganisation.setText("Organisation");

        labelInstitution.setText("Institution");

        labelSchool.setText("School");

        labelHowPublished.setText("How Published");

        labelEPrint.setText("ePrint");

        labelCrossRef.setText("CrossRef");

        labelLicense.setText("License");

        cbCrossRef.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                cbCrossRefActionPerformed(evt);
            }
        });

        lLanguage.setText("Language");

        cbLanguage.setModel(new DefaultComboBoxModel(alLanguageList.toArray()));

        javax.swing.GroupLayout panelLicense_OrganisationsLayout = new javax.swing.GroupLayout(panelLicense_Organisations);
        panelLicense_Organisations.setLayout(panelLicense_OrganisationsLayout);
        panelLicense_OrganisationsLayout.setHorizontalGroup(
            panelLicense_OrganisationsLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(panelLicense_OrganisationsLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(panelLicense_OrganisationsLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(labelSchool)
                    .addComponent(labelEPrint)
                    .addComponent(labelCrossRef)
                    .addComponent(labelLicense))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(panelLicense_OrganisationsLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(tfLicense, javax.swing.GroupLayout.DEFAULT_SIZE, 210, Short.MAX_VALUE)
                    .addComponent(cbCrossRef, javax.swing.GroupLayout.Alignment.TRAILING, 0, 210, Short.MAX_VALUE)
                    .addComponent(tfEPrint, javax.swing.GroupLayout.DEFAULT_SIZE, 210, Short.MAX_VALUE)
                    .addComponent(tfSchool, javax.swing.GroupLayout.DEFAULT_SIZE, 210, Short.MAX_VALUE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(panelLicense_OrganisationsLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(labelInstitution)
                    .addComponent(labelOrganisation)
                    .addComponent(labelHowPublished)
                    .addComponent(lLanguage))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(panelLicense_OrganisationsLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(tfHowPublished, javax.swing.GroupLayout.DEFAULT_SIZE, 195, Short.MAX_VALUE)
                    .addComponent(tfInstitution, javax.swing.GroupLayout.DEFAULT_SIZE, 195, Short.MAX_VALUE)
                    .addComponent(tfOrganisation, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.DEFAULT_SIZE, 195, Short.MAX_VALUE)
                    .addComponent(cbLanguage, 0, 195, Short.MAX_VALUE))
                .addContainerGap())
        );
        panelLicense_OrganisationsLayout.setVerticalGroup(
            panelLicense_OrganisationsLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(panelLicense_OrganisationsLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(panelLicense_OrganisationsLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(tfSchool, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(labelSchool, javax.swing.GroupLayout.PREFERRED_SIZE, 19, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(labelInstitution)
                    .addComponent(tfInstitution, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(panelLicense_OrganisationsLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(panelLicense_OrganisationsLayout.createSequentialGroup()
                        .addGroup(panelLicense_OrganisationsLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                            .addComponent(labelOrganisation)
                            .addComponent(tfOrganisation, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(tfHowPublished, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                    .addGroup(panelLicense_OrganisationsLayout.createSequentialGroup()
                        .addGroup(panelLicense_OrganisationsLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                            .addComponent(labelEPrint)
                            .addComponent(tfEPrint, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addGroup(panelLicense_OrganisationsLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                            .addComponent(labelCrossRef)
                            .addComponent(cbCrossRef, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                            .addComponent(labelHowPublished))))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(panelLicense_OrganisationsLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(panelLicense_OrganisationsLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                        .addComponent(labelLicense)
                        .addComponent(tfLicense, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addComponent(lLanguage))
                    .addComponent(cbLanguage, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addContainerGap(18, Short.MAX_VALUE))
        );

        javax.swing.GroupLayout panelGeneralLayout = new javax.swing.GroupLayout(panelGeneral);
        panelGeneral.setLayout(panelGeneralLayout);
        panelGeneralLayout.setHorizontalGroup(
            panelGeneralLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(panelGeneral_type_key_rev, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
            .addComponent(panelAuthorEditorPublisher, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
            .addComponent(panelYearOtherNumbers, javax.swing.GroupLayout.DEFAULT_SIZE, 600, Short.MAX_VALUE)
            .addComponent(panelLicense_Organisations, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
            .addComponent(panelGeneral_titles, javax.swing.GroupLayout.DEFAULT_SIZE, 600, Short.MAX_VALUE)
        );
        panelGeneralLayout.setVerticalGroup(
            panelGeneralLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(panelGeneralLayout.createSequentialGroup()
                .addComponent(panelGeneral_type_key_rev, javax.swing.GroupLayout.PREFERRED_SIZE, 48, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addComponent(panelGeneral_titles, javax.swing.GroupLayout.PREFERRED_SIZE, 184, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(panelAuthorEditorPublisher, javax.swing.GroupLayout.PREFERRED_SIZE, 187, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(panelYearOtherNumbers, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(panelLicense_Organisations, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addContainerGap())
        );

        jScrollPane1.setViewportView(panelGeneral);

        jTabbedPane1.addTab("General", jScrollPane1);

        panelContent.setPreferredSize(new java.awt.Dimension(460, 360));

        labelURL.setText("URL");

        labelURLDate.setText("Date");

        tfURLDate.setToolTipText("Give date as year, month, day, hour, minute, second (e.g. 20100928120045 - year 210, month Sept., day 28th, hour 12, minutes, 00, sec. 45");

        jLabel2.setText("DOI / ISBN");

        tfIdentifier.setToolTipText("DOI or ISBN to identify the bibliography, can try to import relevant data via internet");

        bIdentifierImport.setText("Import");
        bIdentifierImport.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                bIdentifierImportActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout jPanel3Layout = new javax.swing.GroupLayout(jPanel3);
        jPanel3.setLayout(jPanel3Layout);
        jPanel3Layout.setHorizontalGroup(
            jPanel3Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel3Layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(jPanel3Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(jLabel2)
                    .addComponent(labelURL)
                    .addComponent(labelURLDate))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(jPanel3Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                    .addGroup(jPanel3Layout.createSequentialGroup()
                        .addComponent(tfIdentifier, javax.swing.GroupLayout.PREFERRED_SIZE, 395, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(bIdentifierImport))
                    .addComponent(tfURL)
                    .addComponent(tfURLDate))
                .addContainerGap(31, Short.MAX_VALUE))
        );
        jPanel3Layout.setVerticalGroup(
            jPanel3Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, jPanel3Layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(jPanel3Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel2)
                    .addComponent(tfIdentifier, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(bIdentifierImport))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .addGroup(jPanel3Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(labelURL)
                    .addComponent(tfURL, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(jPanel3Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(labelURLDate)
                    .addComponent(tfURLDate, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addGap(22, 22, 22))
        );

        labelAbstract.setText("Abstract");

        taAbstract.setColumns(20);
        taAbstract.setLineWrap(true);
        taAbstract.setRows(5);
        taAbstract.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                taAbstractMouseClicked(evt);
            }
        });
        taAbstract.addInputMethodListener(new java.awt.event.InputMethodListener() {
            public void inputMethodTextChanged(java.awt.event.InputMethodEvent evt) {
            }
            public void caretPositionChanged(java.awt.event.InputMethodEvent evt) {
                taAbstractCaretPositionChanged(evt);
            }
        });
        taAbstract.addPropertyChangeListener(new java.beans.PropertyChangeListener() {
            public void propertyChange(java.beans.PropertyChangeEvent evt) {
                taAbstractPropertyChange(evt);
            }
        });
        jScrollPane7.setViewportView(taAbstract);

        jLabel5.setText("Tags");

        javax.swing.GroupLayout jPanel5Layout = new javax.swing.GroupLayout(jPanel5);
        jPanel5.setLayout(jPanel5Layout);
        jPanel5Layout.setHorizontalGroup(
            jPanel5Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel5Layout.createSequentialGroup()
                .addGroup(jPanel5Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(labelAbstract)
                    .addComponent(jScrollPane7, javax.swing.GroupLayout.DEFAULT_SIZE, 552, Short.MAX_VALUE)
                    .addGroup(jPanel5Layout.createSequentialGroup()
                        .addComponent(jLabel5)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(tfTags, javax.swing.GroupLayout.DEFAULT_SIZE, 512, Short.MAX_VALUE)))
                .addContainerGap())
        );
        jPanel5Layout.setVerticalGroup(
            jPanel5Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel5Layout.createSequentialGroup()
                .addComponent(labelAbstract)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jScrollPane7, javax.swing.GroupLayout.PREFERRED_SIZE, 233, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addGap(18, 18, 18)
                .addGroup(jPanel5Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel5)
                    .addComponent(tfTags, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addContainerGap(38, Short.MAX_VALUE))
        );

        javax.swing.GroupLayout panelContentLayout = new javax.swing.GroupLayout(panelContent);
        panelContent.setLayout(panelContentLayout);
        panelContentLayout.setHorizontalGroup(
            panelContentLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(panelContentLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(panelContentLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING, false)
                    .addComponent(jPanel3, javax.swing.GroupLayout.Alignment.LEADING, 0, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .addComponent(jPanel5, javax.swing.GroupLayout.Alignment.LEADING, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );
        panelContentLayout.setVerticalGroup(
            panelContentLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(panelContentLayout.createSequentialGroup()
                .addContainerGap()
                .addComponent(jPanel3, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jPanel5, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );

        jScrollPane2.setViewportView(panelContent);

        jTabbedPane1.addTab("Content", jScrollPane2);

        jToolBar1.setFloatable(false);
        jToolBar1.setRollover(true);

        lTitle.setMaximumSize(new java.awt.Dimension(160, 15));
        lTitle.setMinimumSize(new java.awt.Dimension(80, 15));
        lTitle.setPreferredSize(new java.awt.Dimension(160, 15));
        jToolBar1.add(lTitle);

        cbPrivate.setFocusable(false);
        cbPrivate.setHorizontalAlignment(javax.swing.SwingConstants.LEFT);
        cbPrivate.setHorizontalTextPosition(javax.swing.SwingConstants.LEFT);
        cbPrivate.setIconTextGap(0);
        cbPrivate.setMargin(new java.awt.Insets(0, 5, 0, 12));
        cbPrivate.setPreferredSize(new java.awt.Dimension(30, 24));
        cbPrivate.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        cbPrivate.addItemListener(new java.awt.event.ItemListener() {
            public void itemStateChanged(java.awt.event.ItemEvent evt) {
                cbPrivateItemStateChanged(evt);
            }
        });
        jToolBar1.add(cbPrivate);

        bPrivate.setIcon(new javax.swing.ImageIcon(getClass().getResource("/net/geoadel/bibcite/Images/privateKey24.png"))); // NOI18N
        bPrivate.setBorderPainted(false);
        bPrivate.setFocusPainted(false);
        bPrivate.setFocusable(false);
        bPrivate.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        bPrivate.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        bPrivate.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                bPrivateActionPerformed(evt);
            }
        });
        jToolBar1.add(bPrivate);

        bSave.setIcon(new javax.swing.ImageIcon(getClass().getResource("/net/geoadel/bibcite/Images/buttons/bibSave.png"))); // NOI18N
        bSave.setToolTipText("Save bibliography entry");
        bSave.setFocusable(false);
        bSave.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        bSave.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        bSave.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                bSaveActionPerformed(evt);
            }
        });
        jToolBar1.add(bSave);

        bDelete.setIcon(new javax.swing.ImageIcon(getClass().getResource("/net/geoadel/bibcite/Images/buttons/bibDelete.png"))); // NOI18N
        bDelete.setToolTipText("Delete bibliography entry");
        bDelete.setFocusable(false);
        bDelete.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        bDelete.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        bDelete.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                bDeleteActionPerformed(evt);
            }
        });
        jToolBar1.add(bDelete);

        jLabel3.setText("          ");
        jToolBar1.add(jLabel3);

        labelRevision.setText("Revision");
        jToolBar1.add(labelRevision);

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(this);
        this.setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(jToolBar1, javax.swing.GroupLayout.DEFAULT_SIZE, 590, Short.MAX_VALUE)
            .addComponent(jTabbedPane1, javax.swing.GroupLayout.DEFAULT_SIZE, 590, Short.MAX_VALUE)
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addComponent(jToolBar1, javax.swing.GroupLayout.PREFERRED_SIZE, 25, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jTabbedPane1, javax.swing.GroupLayout.DEFAULT_SIZE, 451, Short.MAX_VALUE))
        );
    }// </editor-fold>//GEN-END:initComponents

    /**
     * set all fields to light grey
     * @param evt
     */
    private void cbTypeActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_cbTypeActionPerformed
    }//GEN-LAST:event_cbTypeActionPerformed

    /**
     * if key is allowed to be changed, update the key value with the changed year value
     * @param evt
     */
    private void tfYearCaretUpdate(javax.swing.event.CaretEvent evt) {//GEN-FIRST:event_tfYearCaretUpdate
        if (!cbKeyFixed.isSelected()) {
            BaseN b = new BaseN();
            tfKeyValue.setText(b.updateKeyValueBibliography(tfKeyValue.getText(), tfYear.getText(), firstAuthorLastName, bibKeys));
        }
    }//GEN-LAST:event_tfYearCaretUpdate

    private void taAuthorsCaretUpdate(javax.swing.event.CaretEvent evt) {//GEN-FIRST:event_taAuthorsCaretUpdate
        if (!cbKeyFixed.isSelected()) {
            BaseN b = new BaseN();
            if (taAuthors.getText().contains(" ") & !taAuthors.getText().trim().isEmpty()) {
                String lineOne = "";
                if (taAuthors.getText().contains("\n")) {
                    lineOne = taAuthors.getText().substring(0, taAuthors.getText().indexOf("\n"));
                } else {
                    lineOne = taAuthors.getText();
                }
                // System.out.println("first author name: "+lineOne);
                firstAuthorLastName = lineOne.substring(lineOne.lastIndexOf(" ")).trim();
            }
            tfKeyValue.setText(b.updateKeyValueBibliography(tfKeyValue.getText(), tfYear.getText(), firstAuthorLastName, bibKeys));
        }
    }//GEN-LAST:event_taAuthorsCaretUpdate

    /**
     * if JCheckBox cbKeyFixed was changed, update the noKeyChange value
     * ToDo Replace all uses of noKeyChange with cbKeyFixed.isSelected();
     * @param evt
     */
    private void cbKeyFixedStateChanged(javax.swing.event.ChangeEvent evt) {//GEN-FIRST:event_cbKeyFixedStateChanged
        tfKeyValue.setEditable(!cbKeyFixed.isSelected());
    }//GEN-LAST:event_cbKeyFixedStateChanged

    /**
     * If mouse was double clicked on taEditors, open a bigger editor view
     * @param evt
     */
    private void taAuthorsMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_taAuthorsMouseClicked
        if (evt.isControlDown() & evt.getClickCount() > 1 & !evt.isControlDown()) {
            JFrame frame = new JFrame("Texteditor");
            TextAreaPanel tap = new TextAreaPanel();
            String text = tap.TextAreaPanel(taAuthors);
            frame.add(tap);
            frame.pack();
            frame.setVisible(true);
            frame.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
            taAuthors.setText(text);
        }
    }//GEN-LAST:event_taAuthorsMouseClicked
    /**
     * If mouse was double clicked on taEditors, open a bigger editor view
     * @param evt
     */
    private void taEditorsMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_taEditorsMouseClicked
        if (evt.isControlDown() & evt.getClickCount() > 1) {
            JFrame frame = new JFrame("Texteditor");
            TextAreaPanel tap = new TextAreaPanel();
            String text = tap.TextAreaPanel(taEditors);
            frame.add(tap);
            frame.pack();
            frame.setVisible(true);
            frame.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
            taEditors.setText(text);
        }
    }//GEN-LAST:event_taEditorsMouseClicked

    private void taAddressMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_taAddressMouseClicked
        if (evt.isControlDown() & evt.getClickCount() > 1) {
            JFrame frame = new JFrame("Texteditor");
            TextAreaPanel tap = new TextAreaPanel();
            String text = tap.TextAreaPanel(taAddress);
            frame.add(tap);
            frame.pack();
            frame.setVisible(true);
            frame.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
            taAddress.setText(text);
        }
    }//GEN-LAST:event_taAddressMouseClicked

    private void taAbstractMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_taAbstractMouseClicked
        if (evt.isControlDown() & evt.getClickCount() > 1) {
            JFrame frame = new JFrame("Texteditor");
            TextAreaPanel tap = new TextAreaPanel();
            String text = tap.TextAreaPanel(taAbstract);
            frame.add(tap);
            frame.pack();
            frame.setVisible(true);
            frame.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
            taAbstract.setText(text);
        }
    }//GEN-LAST:event_taAbstractMouseClicked

    private void cbCrossRefActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_cbCrossRefActionPerformed
        // crossref is used in bibtex to fill any fields which are empty in this bibliography with the fields, that are in the cross-referenced bibliography [usefull, if several chapters of a book are on their own but all linked to the main book, so that all it's fields are set]
        // load the cross-referenced bibliography
        // System.out.println("cbCrossRefAction Performed");
        ArrayList<Bibliography> alBibs = dbModel.getBibliographiesNewestVersions();
        Bibliography bibCross = null;
        for (Bibliography bib : alBibs) {
            if (bib.getKey().equals(cbCrossRef.getSelectedItem())) {
                bibCross = bib;
            }
        }

        // go through all fields and fill them with the bibCross fields, if they are empty
        if (bibCross != null) {
            if (tfTitle.getText().isEmpty()) {
                tfTitle.setText(bibCross.getTitle());
            }
            if (tfSubtitle.getText().isEmpty()) {
                tfSubtitle.setText(bibCross.getSubtitle());
            }
            if (tfBooktitle.getText().isEmpty()) {
                System.out.println("Overwriting booktitle with cross reference");
                tfBooktitle.setText(bibCross.getBooktitle());
            }
            if (tfJournal.getText().isEmpty()) {
                tfJournal.setText(bibCross.getJournal());
            }
            if (tfSeries.getText().isEmpty()) {
                tfSeries.setText(bibCross.getSeries());
            }
            if (taAuthors.getText().isEmpty()) {
                taAuthors.setText(bibCross.getAuthorNameAsText());
            }
            if (taEditors.getText().isEmpty()) {
                taEditors.setText(bibCross.getEditorNameAsText());
            }
            if (tfPublisher.getText().isEmpty()) {
                tfPublisher.setText(bibCross.getPublisher());
            }
            if (taAddress.getText().isEmpty()) {
                taAddress.setText(bibCross.getAddress());
            }
            if (tfYear.getText().isEmpty()) {
                tfYear.setText(String.valueOf(bibCross.getYear()));
            }
            if (tfMonth.getText().isEmpty()) {
                tfMonth.setText(String.valueOf(bibCross.getMonth()));
            }
            if (tfVolume.getText().isEmpty()) {
                tfVolume.setText(String.valueOf(bibCross.getVolume()));
            }
            if (tfNumber.getText().isEmpty()) {
                tfNumber.setText(bibCross.getNumber());
            }
            if (tfPages.getText().isEmpty()) {
                tfPages.setText(bibCross.getPages());
            }
            if (tfChapter.getText().isEmpty()) {
                tfChapter.setText(String.valueOf(bibCross.getChapter()));
            }
            if (tfEdition.getText().isEmpty()) {
                tfEdition.setText(String.valueOf(bibCross.getEdition()));
            }
            if (tfSchool.getText().isEmpty()) {
                tfSchool.setText(bibCross.getSchool());
            }
            if (tfInstitution.getText().isEmpty()) {
                tfInstitution.setText(bibCross.getInstitution());
            }
            if (tfEPrint.getText().isEmpty()) {
                tfEPrint.setText(bibCross.getEPrint());
            }
            if (tfOrganisation.getText().isEmpty()) {
                tfOrganisation.setText(bibCross.getOrganization());
            }
            if (tfHowPublished.getText().isEmpty()) {
                tfHowPublished.setText(bibCross.getHowpublished());
            }
            if (tfLicense.getText().isEmpty()) {
                tfLicense.setText(bibCross.getLicense());
            }
            if (tfURL.getText().isEmpty()) {
                tfURL.setText(bibCross.getUrl());
            }
            if (tfURLDate.getText().isEmpty()) {
                tfURLDate.setText(String.valueOf(bibCross.getUrlDate()));
            }
            if (taAbstract.getText().isEmpty()) {
                taAbstract.setText(bibCross.getBibAbstract());
            }
        }
    }//GEN-LAST:event_cbCrossRefActionPerformed

    private void bPrivateActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_bPrivateActionPerformed
        cbPrivate.doClick();
}//GEN-LAST:event_bPrivateActionPerformed

    private void cbPrivateItemStateChanged(java.awt.event.ItemEvent evt) {//GEN-FIRST:event_cbPrivateItemStateChanged
        if (!cbPrivate.isSelected()) {
            jTabbedPane1.remove(accessManager);
        } else {
            if (accessManager == null) {
                accessManager = new AccessManagerGUI(bib);
            }
            jTabbedPane1.add(accessManager, "Access");
        }
    }//GEN-LAST:event_cbPrivateItemStateChanged

    private void tfNumberActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_tfNumberActionPerformed
        // TODO add your handling code here:
    }//GEN-LAST:event_tfNumberActionPerformed

    private void tfEditionActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_tfEditionActionPerformed
        // TODO add your handling code here:
    }//GEN-LAST:event_tfEditionActionPerformed

    private void bIdentifierImportActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_bIdentifierImportActionPerformed
        Bibliography bibImp = new Import_bibTexByISBNDOI().getBibTex(tfIdentifier.getText(), this);
        if (bibImp != null) {
            initData(bibImp);
            // BibCiteGUI.setEditorArea(new BibliographyEditor(bibImp));
        }
    }//GEN-LAST:event_bIdentifierImportActionPerformed

    private void taAbstractCaretPositionChanged(java.awt.event.InputMethodEvent evt) {//GEN-FIRST:event_taAbstractCaretPositionChanged
        // System.out.println("taAbstract changed to: "+taAbstract.getText());
    }//GEN-LAST:event_taAbstractCaretPositionChanged

    private void taAbstractPropertyChange(java.beans.PropertyChangeEvent evt) {//GEN-FIRST:event_taAbstractPropertyChange
        // System.out.println("taAbstract changed to: "+taAbstract.getText());
    }//GEN-LAST:event_taAbstractPropertyChange

    private void tfTitleCaretUpdate(javax.swing.event.CaretEvent evt) {//GEN-FIRST:event_tfTitleCaretUpdate
        lTitle.setText(tfTitle.getText());
    }//GEN-LAST:event_tfTitleCaretUpdate

    private void bDeleteActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_bDeleteActionPerformed
        dbModel.deleteObject(bib.getUUID());
    }//GEN-LAST:event_bDeleteActionPerformed

    private void bSaveActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_bSaveActionPerformed
        saveBibliography();
    }//GEN-LAST:event_bSaveActionPerformed
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton bDelete;
    private javax.swing.JButton bIdentifierImport;
    private javax.swing.JButton bPrivate;
    private javax.swing.JButton bSave;
    private javax.swing.JComboBox cbCrossRef;
    private javax.swing.JCheckBox cbKeyFixed;
    private javax.swing.JComboBox cbLanguage;
    private javax.swing.JCheckBox cbPrivate;
    private javax.swing.JComboBox cbType;
    private javax.swing.JLabel jLabel11;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JLabel jLabel3;
    private javax.swing.JLabel jLabel4;
    private javax.swing.JLabel jLabel5;
    private javax.swing.JPanel jPanel3;
    private javax.swing.JPanel jPanel5;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JScrollPane jScrollPane2;
    private javax.swing.JScrollPane jScrollPane4;
    private javax.swing.JScrollPane jScrollPane5;
    private javax.swing.JScrollPane jScrollPane6;
    private javax.swing.JScrollPane jScrollPane7;
    private javax.swing.JTabbedPane jTabbedPane1;
    private javax.swing.JToolBar jToolBar1;
    private javax.swing.JLabel lLanguage;
    private javax.swing.JLabel lTitle;
    private javax.swing.JLabel labelAbstract;
    private javax.swing.JLabel labelAuthors;
    private javax.swing.JLabel labelBooktitle;
    private javax.swing.JLabel labelChapter;
    private javax.swing.JLabel labelCrossRef;
    private javax.swing.JLabel labelEPrint;
    private javax.swing.JLabel labelEditors;
    private javax.swing.JLabel labelHowPublished;
    private javax.swing.JLabel labelInstitution;
    private javax.swing.JLabel labelJournal;
    private javax.swing.JLabel labelKey;
    private javax.swing.JLabel labelLicense;
    private javax.swing.JLabel labelMonth;
    private javax.swing.JLabel labelNumber;
    private javax.swing.JLabel labelOrganisation;
    private javax.swing.JLabel labelPages;
    private javax.swing.JLabel labelPublisher;
    private javax.swing.JLabel labelRevision;
    private javax.swing.JLabel labelSchool;
    private javax.swing.JLabel labelSeries;
    private javax.swing.JLabel labelSubtitle;
    private javax.swing.JLabel labelTitle;
    private javax.swing.JLabel labelType;
    private javax.swing.JLabel labelURL;
    private javax.swing.JLabel labelURLDate;
    private javax.swing.JLabel labelVolume;
    private javax.swing.JLabel labelYear;
    private javax.swing.JPanel panelAuthorEditorPublisher;
    private javax.swing.JPanel panelContent;
    private javax.swing.JPanel panelGeneral;
    private javax.swing.JPanel panelGeneral_titles;
    private javax.swing.JPanel panelGeneral_type_key_rev;
    private javax.swing.JPanel panelLicense_Organisations;
    private javax.swing.JPanel panelYearOtherNumbers;
    private javax.swing.JTextArea taAbstract;
    private javax.swing.JTextArea taAddress;
    private javax.swing.JTextArea taAuthors;
    private javax.swing.JTextArea taEditors;
    private javax.swing.JTextField tfBooktitle;
    private javax.swing.JTextField tfChapter;
    private javax.swing.JTextField tfEPrint;
    private javax.swing.JTextField tfEdition;
    private javax.swing.JTextField tfHowPublished;
    private javax.swing.JTextField tfIdentifier;
    private javax.swing.JTextField tfInstitution;
    private javax.swing.JTextField tfJournal;
    private javax.swing.JTextField tfKeyValue;
    private javax.swing.JTextField tfLicense;
    private javax.swing.JTextField tfMonth;
    private javax.swing.JTextField tfNumber;
    private javax.swing.JTextField tfOrganisation;
    private javax.swing.JTextField tfPages;
    private javax.swing.JTextField tfPublisher;
    private javax.swing.JTextField tfSchool;
    private javax.swing.JTextField tfSeries;
    private javax.swing.JTextField tfSubtitle;
    private javax.swing.JTextField tfTags;
    private javax.swing.JTextField tfTitle;
    private javax.swing.JTextField tfURL;
    private javax.swing.JTextField tfURLDate;
    private javax.swing.JTextField tfVolume;
    private javax.swing.JTextField tfYear;
    // End of variables declaration//GEN-END:variables
}
