/* bibcite is a bibliography and citation library with exporting
and importing capabilities to bibTex format

Copyright (C) 2010  Georg Adelmann - mail@geoadel.net

This file is part of bibcite.

bibcite free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>. */
package net.geoadel.bibcite.Tools;

import java.util.ArrayList;

/**
 * Concatenates a letter to a string to make it unique - the letter chosen is the next one in the alphabet
 * theo1999 -> theo1999a -> theo1999b .... theo1999z -> theo1999aa -> theo1999az -> theo1999ba
 *
 * @GPLStatus   clean
 * @DocumentationStatus finished
 * @author  Georg Adelmann
 */
public class BaseN {

    static int base = 26; // the numerical base, should be adjusted to the charset - in this case standard latin alphabet of ASCII is used, so 26 is the correct value to go from a till z

    /**
     * get the trailing character, that is thought to make the key unique
     * @param number    iterator, that gets up, each time, getBaseNChar returns a char, that NOT YET creates a unique key
     * @return  character, that corresponds to the number modulo base
     */
    private static String getBaseNChar(int number) {
        String trailingCharacter = ""; // needs to be outside the loop, because if the loop runs several times, additional characters will be added at it's end
        while (number > 0) { // while number bigger than zero
            int remainder = number % base; // number modulo the given base, leaves the integer remainder of the division, thus indicating, which letter to use as a trail
            trailingCharacter = (char) (remainder + '`') + trailingCharacter; // +'`' is the char/numerical before 'a' in the ASCII table; together with the trailing char offset, that shifts ASCII index, so that it starts with small letter a and follows b,c,d,e,... - for the latin alphabet, base number 26 is chosen, for other alphabets, this probably should be changed according with a fitting charset
            number = number / base; // set the number to number / base, needed in case there are so many keys already, that the first run of length base does not lead to a unique key. In this case, another letter has to be added. This assignment controls this behaviour.
        }
        return trailingCharacter; // return the trailing char
    }

    /**
     * Updates the value of the Bibliography key (should be called in case of changes of components, which
     * add to the key-value - specifically tableAuthor, year, title
     * @param key   the key, that is used, if no collisions occur
     * @param year  the year of the entry
     * @param authorLastName    the last name of the first author
     * @param bibKeys   already existing keys
     * @return  the unique key String
     */
    public String updateKeyValueBibliography(String key, String year, String authorLastName, ArrayList<String> bibKeys) {
        String newKey = authorLastName;

        if (year.length() > 0) {
            newKey = newKey.concat(year);
        }
        int baseKey = newKey.length(); // stores the basekey-length, which helps adjusting the end, if it is not unique
        // check, if the key is unique [then he is NOT in the bibKeys list] and that this entry is new [if entry not new, the key should never be changed]
        if (bibKeys != null && isDuplicate(newKey, bibKeys)) {
            int i = 0;
            while (true) { // while loops until break is called
                newKey = newKey.substring(0, baseKey).concat(getBaseNChar(i)); // as long as newKey is not unique, toBase26(i) will give new combinations of chars to add
                if (!isDuplicate(newKey, bibKeys)) { // if newKey is not a duplicate, the loop is aborted
                    break;
                }
                i++;
            }
        }
        return newKey;
    }

    public static String updateKeyValueQuotation(String pages, String bibKey, ArrayList<String> quoteKeys) {
        String newKey = bibKey+"::"+pages;
                int baseKey = newKey.length(); // stores the basekey-length, which helps adjusting the end, if it is not unique
        // check, if the key is unique [then he is NOT in the bibKeys list] and that this entry is new [if entry not new, the key should never be changed]
        if (quoteKeys != null && isDuplicate(newKey, quoteKeys)) {
            // System.out.println("Key " + newKey + " already exists");
            int i = 0;
            while (true) { // while loops until break is called
                newKey = newKey.substring(0, baseKey).concat(getBaseNChar(i)); // as long as newKey is not unique, toBase26(i) will give new combinations of chars to add
                if (!isDuplicate(newKey, quoteKeys)) { // if newKey is not a duplicate, the loop is aborted
                    break;
                }
                i++;
            }
        }
        return newKey;
    }

    /**
     * Updates the value of the Bibliography key (should be called in case of changes of components, which
     * add to the key-value - specifically tableAuthor, year, title
     * @param key   the key, that is used, if no collisions occur
     * @param year  the year of the entry
     * @param name    the last name of the first author
     * @param authorKeys   already existing keys
     * @return  the unique key String
     */
    public static String updateKeyValueAuthor(String key, String year, String name, ArrayList<String> authorKeys) {

        String newKey = name;
        if (year != null) { // add year, if given and length 4 digits
            if (year.length() == 4) {
                newKey = newKey + " " + year;
            } else {
                if (year.length() > 4) {
                    newKey = newKey + " " + year.substring(0, 4);
                }
            }
        }


        int baseKey = newKey.length(); // stores the basekey-length, which helps adjusting the end, if it is not unique
        // check, if the key is unique [then he is NOT in the bibKeys list] and that this entry is new [if entry not new, the key should never be changed]
        if (authorKeys != null && isDuplicate(newKey, authorKeys)) {
            // System.out.println("Key " + newKey + " already exists");
            int i = 0;
            while (true) { // while loops until break is called
                newKey = newKey.substring(0, baseKey).concat(getBaseNChar(i)); // as long as newKey is not unique, toBase26(i) will give new combinations of chars to add
                if (!isDuplicate(newKey, authorKeys)) { // if newKey is not a duplicate, the loop is aborted
                    break;
                }
                i++;
            }
        }
        return newKey;
    }

    /**
     * Short check, whether the key is duplicate
     * @param key   key
     * @param keys   list of keys
     * @return  boolean, whether key is already in bibKeys ArrayList
     */
    private static Boolean isDuplicate(String key, ArrayList<String> keys) { // checks, whether the ArrayList biKeys contains String key
        return keys.contains(key);
    }
}
